#!/bin/bash
#copyfile.linux.sh
#copys a bhuman distribution to a robot

DEBUG=0

debug()
{
  if [ $DEBUG == 1 ]; then
    echo $@ 
  fi
}

usage()
{
  echo "usage:   $PROG Nao[Lib](Debug|Optimized|Release) <ipaddress> {options}"
  echo options:
  echo   "-l <location> set location"
  echo   "-t <color>    set team color to blue or yellow"
  echo   "-p <number>   set player number"
  echo   -s            copy binary with debug symbols
  echo   -z            generate deploy list instead of copying
  echo   -d            delete remotecache directory
  echo example: $PROG NaoOptimized 134.102.204.229 -p 1
  exit 1
}

initvars()
{
  debug $SOURCE
  CONF=$1
  DRIVE=$2
  PLAYER=
  DELETE=0
  TEAM=
  LOCATION=
  REMOTE=
  OPTIONS=-u
  TARGET=$DRIVE
  STRIPPED=-stripped
  SOURCE=\\$HOSTNAME\\$GT_DIR\\$CONF
}

parseOptions()
{
  shift
  while true; do
    case $2 in
      "")
        break
        ;;
      "-p" | "/p")
        shift
        PLAYER=$2
        ;;
      "-l" | "/l")
        shift
        LOCATION=$2
        ;;
      "-z" | "/z")
        OPTIONS=
        TARGET=../Build/copytmp
        ;;
      "-t" | "/t")
        shift
        TEAM=$2
        ;;
      "-s" | "/s")
        STRIPPED=
        ;;
      "-d" | "/d")
        DELETE=1
        ;;
      *)
        echo "unknown parameter $2"
        ;;
    esac
    shift
  done
}

copy()
{
  sed -e 's/\\//' exclude.txt > .exclude.tmp
  rsync $OPTIONS --exclude-from=.exclude.tmp --verbose $@  | sed -e '/sent.*bytes.*received.*bytes.*bytes.*sec/d' -e '/total size is.*speedup/d' -e '/^$/d' -e '/sending incremental file list/d'
  rm .exclude.tmp
}

rcopy()
{
  sed -e 's/\\//' excludesvn.txt > .excludesvn.tmp
  rsync $OPTIONS -r --exclude-from=.excludesvn.tmp --verbose $1 $2  | sed -e '/sent.*bytes.*received.*bytes.*bytes.*sec/d' -e '/total size is.*speedup/d' -e '/^$/d' -e '/sending incremental file list/d'
  rm .excludesvn.tmp
}

appendSettingsCfg()
{
  echo $1 >> $TARGET/settings.cfg
  echo $1
}

copyremotely()
{
  REMOTE=$DRIVE
  TARGET=../Build/remotecache
  if [ $DELETE -eq 1 ] ; then
    rm -r $TARGET
    ssh -i Keys/id_rsa_nao root@$REMOTE rm -rf /media/userdata/Config/* 2> /dev/null
  fi

  mkdir -p $TARGET

  #check whether it was written from the same source
  SOURCE=$SOURCE$STRIPPED
  CFGFILE="$TARGET/config.cfg"
  if [ -a $CFGFILE ]; then
    if [ `cat $CFGFILE` != $SOURCE ]; then
      echo Error: drive $DRIVE: contains a different configuration or was written from a different checkout. Use -d.
      exit 1
    fi
  fi

  if [ `ssh -i Keys/id_rsa_nao root@$REMOTE "cat /media/userdata/Config/config.cfg 1> /dev/null 2> /dev/null" 2> /dev/null ; echo $?` == "0" ] ; then
    if [ `ssh -i Keys/id_rsa_nao root@$REMOTE cat /media/userdata/Config/config.cfg 2> /dev/null` != $SOURCE ]; then
      echo Error: $REMOTE:/media/userdata/Config contains a different configuration or was written from a different checkout. Use -d.
      exit 1
    fi
  fi
  echo $SOURCE > $CFGFILE

  debug "read settings.cfg"
  #read settings.cfg
  [ -z $PLAYER ] && PLAYER="`egrep ^playerNumber\  settings.cfg | awk '{print $2}' | sed -e 's/\r//'`"
  [ -z $LOCATION ] && LOCATION="`egrep ^location\  settings.cfg | awk '{print $2}' | sed -e 's/\r//'`"
  [ -z $TEAM ] && TEAM="`egrep ^teamColor\  settings.cfg | awk '{print $2}' | sed -e 's/\r//'`"
  
  debug "copy libbhuman.so bhuman"
  #copyfile.cmd:copy
  if [ $OPTIONS ]; then
    [ -a ../Build/Linux/$CONF/libbhuman$STRIPPED.so ] && copy ../Build/Linux/$CONF/libbhuman$STRIPPED.so $TARGET/libbhuman.so
    [ -a ../Build/Linux/$CONF/bhuman$STRIPPED ] && copy ../Build/Linux/$CONF/bhuman$STRIPPED $TARGET/bhuman
    [ -a ../Util/libnao/libnao.so ] && copy ../Util/libnao/libnao.so $TARGET/libnao.so
  fi

  debug "copyconfig"
  #copyfile.cmd:copyconfig
  copy * $TARGET
  [ ! -d $TARGET/Locations ] && mkdir $TARGET/Locations
  debug "\$LOCATION $LOCATION"
  rcopy Locations/$LOCATION $TARGET/Locations/
  rcopy Robots $TARGET/
  rcopy Processes $TARGET/
  rcopy Xabsl $TARGET/
  rcopy Sounds $TARGET/
  [ $OPTIONS ] || echo settings.cfg

  debug "settings"
  echo "----SETTINGS----"
  #copyfiles.cmd:settings
  cat settings.cfg | sed -e 's/\r//' > $TARGET/settings.cfg
  appendSettingsCfg "model nao"
  appendSettingsCfg "location $LOCATION"
  appendSettingsCfg "teamColor $TEAM"
  appendSettingsCfg "playerNumber $PLAYER"

  if [ $OPTIONS ]; then
    debug "rsync to robot"
    echo "----SENDING FILES TO ROBOT----"
    pushd ../Build/remotecache > /dev/null
    rsync -rve "ssh -i ../../Config/Keys/id_rsa_nao" * root@$REMOTE:/media/userdata/Config  | sed -e '/sent.*bytes.*received.*bytes.*bytes.*sec/d' -e '/total size is.*speedup/d' -e '/^$/d'
    popd > /dev/null
  fi
}

echo "--- B-Human Copyfiles ---"

#test correct call
if [ -z $1 ] || [ -z $2 ] ; then usage ; fi
#determine GT_DIR
if [[ $0 == /* ]] ; then #if $0 starts with /
  GT_DIR=`dirname $0`
  GT_DIR=`dirname $GT_DIR`
else
  if [[ $0 == ./* ]] ; then #if $0 starts with ./
    GT_DIR=`pwd`
    GT_DIR=`dirname $GT_DIR`
  else
    GT_DIR="`pwd`/`dirname $0`"
    GT_DIR=`dirname $GT_DIR`
  fi
fi

pushd $GT_DIR/Config > /dev/null

initvars $1 $2
parseOptions $@

debug "player $PLAYER location $LOCATION target $TARGET team $TEAM stripped $STRIPPED"

copyremotely

popd > /dev/null
