
#include <vector>
#include <string>
#include <sstream>
#include "../InternalData.h"
#include "../Transformation.h"
#include "../ImageProcessingData/blob.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "BallPerceptorGT.h"


BallPerceptorGT::BallPerceptorGT(const std::string& name, const Transformation& transformation,
                                 const std::vector<Blob>& orangeBlobs, const unsigned& timeStamp,
                                 BallPerceptGT& ballPercept) :
  name(name), ballPercept(ballPercept), orangeBlobs(orangeBlobs), transformation(transformation), timeStamp(timeStamp)
{
}

void BallPerceptorGT::execute()
{
  DECLARE_DEBUG_DRAWING("ground truth:ball perceptor:ball", "drawingOnImage", "The ball percept.");
  bool found_good_ball(false);
  std::vector<Blob>::const_iterator blob = orangeBlobs.begin();
  std::vector<Blob>::const_iterator best = blob;

  while(blob != orangeBlobs.end())
  {
    if (!found_good_ball || blob->size > best->size)
    {
      best = blob;
    }
    found_good_ball = true;
    blob++;
  }
  
  if (found_good_ball)
  {
    ballPercept.x = (*best).center_x;
    ballPercept.y = (*best).center_y; 
    ballPercept = BallPerceptGT(transformation.toField(ballPercept.x, ballPercept.y, ObjectHeights::BALL_HEIGHT));
    ballPercept.timeStamp = timeStamp;
    CIRCLE("ground truth:ball perceptor:ball", (*best).center_x, (*best).center_y, 20, 1,
           Drawings::ps_solid, ColorClasses::orange, Drawings::bs_solid, ColorRGBA(255,128,0,100));
    std::stringstream s;
    s << "BallPercept: (" << ballPercept.x << ", " << ballPercept.y <<")";
    TIP("ground truth:ball perceptor:ball", (*best).center_x, (*best).center_y, 20, s.str());
  }
}
