#ifndef TEAM_MODULE_H
#define TEAM_MODULE_H

#include "RobotProfile.h"


class RobotPerceptorGT 
{
public:

  RobotPerceptorGT(const std::string& name, const FireWireImage& image, const Transformation& transformation,
    const std::vector<Blob>& yellowBlobs, const std::vector<Blob>& blueBlobs,
    RobotPerceptListGT& yellowRobots, RobotPerceptListGT& blueRobots);

  ~RobotPerceptorGT();


  /**
  * Creates robot percepts from a list of blobs, both available as vectors in
  * the image_processor_data structure.
  */
  virtual void execute();
  virtual void loadConfig();

protected:

  /**
  * Take the blobs of the colors to be tracked
  */
  void create_robot_profiles();

  bool distanceOK(int x, int y);

  RobotPerceptListGT& yellowRobots;
  RobotPerceptListGT& blueRobots;
  const std::vector<Blob>& yellowBlobs;
  const std::vector<Blob>& blueBlobs;

  const Transformation& transformation;
  const FireWireImage& image;

  int scan_radius; //!< Scan radius for profile detection in pixel 
  int scan_points; //!< Number of points the scan circle is created of
  int securityDistance; //!< Minimum distance of a blob center to the border

  std::string name;
  std::vector<RobotProfile*> profiles;

  RobotPatterns patterns;
};

#endif // TEAM_MODULE_H
