#ifndef ROBOT_PROFILE_H
#define ROBOT_PROFILE_H


/**
 * This class implements a robot profile which is used to identify the
 * orientation and ID of a given robot using the colored pattern on top of the
 * robot.
 */
class RobotProfile
{
public:
    /**
     * Creates a new profile by doing all the necessary steps to identify the ID
     * and rotation.
     */
    RobotProfile(const Blob& blob, const FireWireImage& image, const Transformation& trans,
                 int scan_radius, RobotPatterns& patterns);
    
    TeamColor get_color()               { return team_color;     }    
    double get_x()                      { return center.x;       }
    double get_y()                      { return center.y;       }
    Vector2<double> get_center()        { return center;         }
    double get_rotation()               { return rotation;       }
    unsigned short get_id()             { return id;             }

protected:
    
    void findInitialRotation();
    void findInitialRotationThomas();

    void findRealRotationAndId();

    double minRadius, maxRadius;
    Vector2<double> center;       //!< Center of profile
    unsigned short id;            //!< ID of this profile
    double rotation;              //!< Rotation of this profile
    const FireWireImage& image;   //!< Image containing this profile 
    int scan_radius;              //!< Radius from center to scan circle    
    TeamColor team_color;         //!< Team color of this profile
    
private:
    /**
     * mini wrapper for call to transformation.toImage
     */
    Vector2<double> t(const Vector2<double>& vec) const;
    
    RobotPatterns& patterns;
    
    const Transformation& trans; //!< Underlying transformation for this profile
    
    ObjectHeights::HeightClass height; //!< Height for transformation

    RGB_Pixel getRGBPixel(const FireWireImage& image, int x, int y) const;

    RGB_Pixel getRGBPixelInterpolated(const FireWireImage& image, int x, int y) const;
};

#endif // ROBOT_PROFILE_H
