#ifndef BLOB_DETECTOR_MODULE_H
#define BLOB_DETECTOR_MODULE_H

#include "../ImageProcessingData/blob.h"
#include "../ImageProcessingData/color_class_rle_image.h"
#include <vector>

struct Blob_Constraints
{
  Blob_Constraints()
  {
    constraint[TEAM].min_size = 10;
    constraint[TEAM].max_size = 100;
    constraint[TEAM].min_width = 1.;
    constraint[TEAM].xy_ratio = 0.7;
    constraint[BALL].min_size = 10;
    constraint[BALL].max_size = 100;
    constraint[BALL].min_width = 1.;
    constraint[BALL].xy_ratio = 0.7;
  }

  enum Constraint_Type
  {
    BALL = 0,
    TEAM,
    SIZE_OF_CONSTRAINT
  };

  bool check_constraints(const Blob& b, Constraint_Type ct)
  {
    const constraint_t& c = constraint[ct];
    return (   b.size >= c.min_size \
      && b.size <= c.max_size \
      && (b.small_length / b.large_length >= c.xy_ratio)
      && b.small_length >= c.min_width);
  }

  struct constraint_t
  {
    double min_size;
    double max_size;
    double min_width;
    double xy_ratio;
  };

  constraint_t constraint[SIZE_OF_CONSTRAINT];
};


class Blob_Detector_Module 
{
public:
  Blob_Detector_Module(const std::string& name, Color_Class_RLE_Image& rleImage,
    std::vector<Blob>& yellowBlobs, std::vector<Blob>& orangeBlobs, std::vector<Blob>& blueBlobs);

  virtual void execute();

  /** Contains user interaction and drawing stuff*/
  void configure();

  virtual void saveConfig();
  virtual void loadConfig();

protected:
  void group_regions();
  void compress_path(Run* run);
  void unite_regions(Run* region1, Run* region2);
  void find_potential_blobs();
  void check_and_convert_blobs();

  const std::string& name;
  Color_Class_RLE_Image& rleImage;
  std::vector<Blob>& yellow_blobs;
  std::vector<Blob>& blue_blobs;
  std::vector<Blob>& orange_blobs;
  std::vector<Blob> potential_blobs;
  Blob_Constraints blob_constraints;
};

#endif // BLOB_DETECTOR_MODULE_H
