#ifndef RLE_COMPRESSION_MODULE_H
#define RLE_COMPRESSION_MODULE_H

#include "../ImageProcessingData/colors.h"
#include "../ImageProcessingData/color_class_rle_image.h"
#include <string.h>
#include "../InternalData.h"


struct RGB_Pixel_Settings : public Streamable
{
public:
  RGB_Pixel_Settings()
  {
    yellow_min.r = yellow_min.g = yellow_min.b = 42;
    yellow_max = blue_min = blue_max = orange_min = orange_max = yellow_min;
  }
  
  virtual void serialize(In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN();
      STREAM(yellow_min);
      STREAM(yellow_max);
      STREAM(orange_min);
      STREAM(orange_max);
      STREAM(blue_min);
      STREAM(blue_max);
    STREAM_REGISTER_FINISH();
  }

  RGB_Pixel yellow_min;
  RGB_Pixel yellow_max;
  RGB_Pixel orange_min;
  RGB_Pixel orange_max;
  RGB_Pixel blue_min;
  RGB_Pixel blue_max;
} ;


class RGBLUT64
{
public:
  unsigned char colorClass[64][64][64];

  RGBLUT64()
  {
    memset(&colorClass[0], COLOR_CLASS_NONE, 64*64*64);
  }

  void update(const RGB_Pixel_Settings& s);
};


/**
* This module creates a color class RLE image 
*/
class RLE_Compression_Module
{
public:
  /**
  * Constructs a new rle compression module with the given parameters.
  *
  * @param  name      The name of the configuration
  * @param  image     The image
  * @param  rleImage  The resulting compressed image
  */
  RLE_Compression_Module(const std::string& name,
                         FireWireImage& image,
                         Color_Class_RLE_Image& rleImage);   

  ~RLE_Compression_Module();

  /**
  * Executes the rle compression onto the current image in the image
  * processor data structure.
  */
  void execute();

  /** Contains user interaction and drawing stuff*/
  void configure();

  /**
  * Loads previously used classification values from the filessystem.
  */
  void loadConfig();

  /**
  * Saves the current classification values to the filessystem.
  */
  void saveConfig();

  /** Prints the RGB values of a pixel to the console*/
  void printPixelValue(int x, int y);

protected:
  /**
  * Classifies the given pixel according to the current classification values
  * into a color class.
  *
  * @param  pixel  the pixel to be classified
  * @return        the color class this pixel belongs to
  */
  Color_Class classify(unsigned char*& pixel) const;

  FireWireImage& image;        //!< The original image
  Color_Class_RLE_Image& rleImage;   //!< The rle image to save the runs in
  const std::string name;            //!< The name of the current config
  RGBLUT64* rgbLUT;                  //!< A look-up table for color classes
  RGB_Pixel_Settings pixelSettings;  //!< The current color class cubes in RGB space
};

#endif
