#ifndef BLOB_GT_H
#define BLOB_GT_H 

#include <map>

#include "colors.h"

struct Blob
{
    // Bounding Box
    double x;
    double y;
    double w;
    double h;
    // Gravity Center
    double center_x;
    double center_y;
    // Size of Blob
    double size;
    // Inertia
    double I, I_x, I_y, I_xx, I_xy, I_yy;
    // Orientation of "Main Inertia-Axis"
    double main_axis;
    
    double small_length;
    double large_length;

    Color_Class color;

    Blob(): x(0), y(0), w(0), h(0), size(0),
            I(0), I_x(0), I_y(0),I_xx(0), I_xy(0), I_yy(0)
    {};
};

struct Blob_With_Angle : public Blob
{
    double angle;
    bool operator< (const Blob_With_Angle& other) const
    {return this->angle < other.angle;};
};


// Sort functors 
struct sort_blob_x : public std::binary_function<const Blob&, const Blob&, bool> 
{
    bool operator()(const Blob& a, const Blob& b)
    {
        return a.center_x < b.center_x;
    }
};

struct sort_blob_y : public std::binary_function<const Blob&, const Blob&, bool> 
{
    bool operator()(const Blob& a, const Blob& b)
    { 
        return a.center_y < b.center_y;
    }
};


#endif /* BLOB_GT_H */
