#ifndef COLOR_CLASS_RLE_IMAGE_H_
#define COLOR_CLASS_RLE_IMAGE_H_

#include <vector>
#include "colors.h"


/**
 * A run of a single color in the rle image.
 */
struct Run
{
    int x_start;         //!< The start index
    int x_end;           //!< The end index
    int y;               //!< The line index
    unsigned int region; //!< The id of the region
    Run* parent;         //!< A reference to the parent of this run
    Color_Class color;   //!< The color class of this run
};

/**
 * A rle image consisting of a number of runs.
 */
class Color_Class_RLE_Image
{
public:
    /**
     * Constructs a new rle image which allocates memory for a number of runs.
     *
     * @param  initial_num_of_elements  the number of runs to reserve memory for
     */
    Color_Class_RLE_Image(int initial_num_of_elements = 500)
    {
        data.reserve(initial_num_of_elements);
    }
    
    /**
     * Deletes all runs from this image.
     */
    void reset()
    {
        data.clear();
    }

    /**
     * Copy constructor.
     *
     * @param  other  the image to be copied
     */
    void operator=(const Color_Class_RLE_Image& other)
    {
        data = other.data;
    }
    
    std::vector<Run> data; //!< Vector containing the individual runs
};

#endif // COLOR_CLASS_RLE_IMAGE_H_











