/**
* @file ControllerQt/Views/FieldView.h
*
* Declaration of class FieldView
*
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#ifndef __FIELDVIEW_H__
#define __FIELDVIEW_H__

#include <SimRobotCore/Controller/DirectView.h>

class RobotConsole;
class QWidget;

/**
* @class FieldView 
* 
* A class to represent a view displaying debug drawings in field coordinates.
*
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/
class FieldView : public DirectView
{
public:
  /**
  * Constructor.
  * @param console The console object.
  * @param name The name of the view.
  */
  FieldView(RobotConsole& console, const std::string& name);

private:
  /**
  * The method returns a new instance of a widget for this direct view.
  * The caller has to delete this instance. (Qt handles this)
  * @return The widget.
  */
  QWidget* createWidget();

  /**
  * The function is called to let the direct view update the widget. Do so if necessary.
  * @param widget The widget
  */
  void updateWidget(QWidget& widget);

  RobotConsole& console; /**< A reference to the console object. */
  const std::string name; /**< The name of the view. */
};

#endif //__FIELDVIEW_H__
