/**
* @file ControllerQt/Views/GroundTruthView.h
*
* Declaration of class GroundTruthView
*
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
* @author Colin Graf
*/

#ifndef __GROUNDTRUTHVIEW_H__
#define __GROUNDTRUTHVIEW_H__

#include <SimRobotCore/Controller/DirectView.h>

class RobotConsole;
class QWidget;

/**
* @class GroundTruthView 
* 
* A class to represent a view displaying camera images and overlaid debug drawings.
*
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/
class GroundTruthView : public DirectView
{

public:
  /**
  * Constructor.
  * @param console The console object.
  * @param name The name of the view.
  * @param segmented Ignored.
  */
  GroundTruthView(RobotConsole& console, const std::string& name, bool segmented);

private:

  /**
  * The method returns a new instance of a widget for this direct view.
  * The caller has to delete this instance. (Qt handles this)
  * @return The widget.
  */
  QWidget* createWidget();

  /**
  * The function is called to let the direct view update the widget. Do so if necessary.
  * @param widget The widget
  */
  void updateWidget(QWidget& widget);

  RobotConsole& console; /**< A reference to the console object. */
  const std::string name; /**< The name of the view. */
};

#endif //__GROUNDTRUTHVIEW_H__
