/**
* @file ControllerQt/Views/SensorView.h
*
* Declaration of class SensorView
*
* @author of original sensorview <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
* @author Jeff
*/

#ifndef __SensorView_h_
#define __SensorView_h_

#include <SimRobotCore/Controller/DirectView.h>
#include "../RobotConsole.h"

class SensorData;
class JointData;
/**
* @class SensorView 
* 
* A class to represent a view with information about the sensor values.
*
* @author of the original sensorview <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
* @author Jeff
*/
class SensorView : public DirectView
{
    RobotConsole& console; /**< A reference to the console object. */
    const JointData& jointData; /**< The most current joint angle data. */
    const SensorData&  sensorData; /**< The most current sensor data. */
    QWidget *widget;

    QWidget* createWidget();
    void updateWidget(QWidget& widget);

  public:
    /**
    * Constructor.
    * @param c The console object.
    * @param j The most current joint data to be visualized.
    * @param s The most current sensor data to be visualized.
    */
    SensorView(RobotConsole& c, const JointData& j, const SensorData& s): console(c),
                                                                          jointData(j),
                                                                          sensorData(s){};


};

#endif //__SensorView_h_
