/** 
* @file BH2009FieldSymbols.h
*
* Declaration of class BH2009FieldSymbols.
*/

#ifndef __BH2009FieldSymbols_h_
#define __BH2009FieldSymbols_h_

#include "../../Symbols.h"
#include "Representations/Configuration/FieldDimensions.h"

/**
* The Xabsl symbols that are defined in "field_symbols.xabsl"
*/ 
class BH2009FieldSymbols : public Symbols
{
public:
  /* Constructor */
  BH2009FieldSymbols(FieldDimensions const& fieldDimensions) : fieldDimensions(fieldDimensions)
  {
    theInstance = this;        
  }

  PROCESS_WIDE_STORAGE_STATIC BH2009FieldSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** registers the symbols at the engine */
  void registerSymbols(xabsl::Engine& engine);

  void init()
  {
    ballRadius = (double) theInstance->fieldDimensions.ballRadius;
    xPosOwnGoal = (double) theInstance->fieldDimensions.xPosOwnGoal;
    xPosOwnGroundline = (double) theInstance->fieldDimensions.xPosOwnGroundline;
    xPosOpponentGroundline = (double) theInstance->fieldDimensions.xPosOpponentGroundline;
    yPosLeftSideline = (double) theInstance->fieldDimensions.yPosLeftSideline;
    yPosRightSideline = (double) theInstance->fieldDimensions.yPosRightSideline; 
    yPosLeftFieldBorder = (double) theInstance->fieldDimensions.yPosLeftFieldBorder;
    yPosRightFieldBorder = (double) theInstance->fieldDimensions.yPosRightFieldBorder; 
    centerCircleRadius = (double) theInstance->fieldDimensions.centerCircleRadius;
    xPosOwnPenaltyArea = (double) theInstance->fieldDimensions.xPosOwnPenaltyArea;
    xPosOpponentPenaltyArea = (double) theInstance->fieldDimensions.xPosOpponentPenaltyArea;
    yPosLeftPenaltyArea = (double) theInstance->fieldDimensions.yPosLeftPenaltyArea;
    yPosRightPenaltyArea = (double) theInstance->fieldDimensions.yPosRightPenaltyArea;
  };

private:
  FieldDimensions const& fieldDimensions;
  double ballRadius;
  double xPosOwnGoal;
  double xPosOwnGroundline;
  double xPosOpponentGroundline;
  double yPosLeftSideline;
  double yPosRightSideline; 
  double yPosLeftFieldBorder;
  double yPosRightFieldBorder; 
  double centerCircleRadius;
  double xPosOwnPenaltyArea;
  double xPosOpponentPenaltyArea;
  double yPosLeftPenaltyArea;
  double yPosRightPenaltyArea;
};


#endif // __FieldSymbols_h_
