/** 
* @file BH2009GameSymbols.h
*
* Declaration of class BH2009GameSymbols.
*
* @author Judith Mller
*/

#ifndef __BH2009GameSymbols_h_
#define __BH2009GameSymbols_h_

#include "Representations/Infrastructure/RobotInfo.h"
#include "Representations/Infrastructure/TeamInfo.h"
#include "Representations/Infrastructure/GameInfo.h"
#include "../../Symbols.h"

/**
* The Xabsl symbols that are defined in "game_symbols.xabsl"
*
* @author Max Risler
* @author Judith Mller
*/ 
class BH2009GameSymbols : public Symbols
{
public:
  /*
  * Constructor.
  * @param robotInfo A reference to the RobotInfo.
  * @param ownTeamInfo A reference to the OwnTeamInfo.
  * @param opponentTeamInfo A reference to the OpponentTeamInfo.
  * @param gameInfo A reference to the GameInfo.
  */
  BH2009GameSymbols(RobotInfo const& robotInfo, OwnTeamInfo& ownTeamInfo, GameInfo& gameInfo) : 
    robotInfo(robotInfo), 
    ownTeamInfo(ownTeamInfo),
    gameInfo(gameInfo),
    startup(true)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC BH2009GameSymbols* theInstance;

  /** A reference to the RobotInfo */
  RobotInfo const& robotInfo;

  OwnTeamInfo& ownTeamInfo;
  GameInfo& gameInfo;

  bool startup;

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  static int getOwnTeamColor(){return (int)theInstance->ownTeamInfo.teamColor;}
  static void setOwnTeamColor(int color){theInstance->ownTeamInfo.teamColor = color;}

  static int getOppTeamColor()
  {
    if ((int)theInstance->ownTeamInfo.teamColor == TEAM_YELLOW) return TEAM_BLUE;
    else return TEAM_YELLOW;
  }

  static int getPlayerNumber(){return theInstance->robotInfo.number;}

  static int getPenalty(){return (int)theInstance->robotInfo.penalty;}

  static double getPenaltyRemainingTime(){return (double)theInstance->robotInfo.secsTillUnpenalised;}

  static int getState(){return (int)theInstance->gameInfo.state;}
  static void setState(int state){theInstance->gameInfo.state = state;}

  static int getKickoffTeam(){return (int)theInstance->gameInfo.kickOffTeam;}
  static void setKickoffTeam(int kickoff){theInstance->gameInfo.kickOffTeam = kickoff;}

  static double getOwnScore(){return (double)theInstance->ownTeamInfo.score;}

  static double getRemainingTime(){return (double)theInstance->gameInfo.secsRemaining;}

  static bool getKeeperPenalized(){return theInstance->ownTeamInfo.players[0].secsTillUnpenalised > 0;}

  static bool getDefenderPenalized(){return theInstance->ownTeamInfo.players[1].secsTillUnpenalised > 0;}

  static bool getSupporterPenalized(){return theInstance->ownTeamInfo.players[2].secsTillUnpenalised > 0;}

  static bool getStrikerPenalized(){return theInstance->ownTeamInfo.players[3].secsTillUnpenalised > 0;}
};


#endif // __GameSymbols_h_

