/** 
* @file BH2009GoalSymbols.cpp
*
* Implementation of class BH2009GoalSymbols.
*
* @author Tim Laue
*/

#include "BH2009GoalSymbols.h"

PROCESS_WIDE_STORAGE BH2009GoalSymbols* BH2009GoalSymbols::theInstance = 0;


void BH2009GoalSymbols::registerSymbols(xabsl::Engine& engine)
{
  // "goal.left_angle_to_free_part"
  engine.registerDecimalInputSymbol("goal.left_angle_to_free_part", &getAngleToLeftEnd);
  // "goal.right_angle_to_free_part"
  engine.registerDecimalInputSymbol("goal.right_angle_to_free_part", &getAngleToRightEnd);
  // "goal.angle_to_center_of_free_part"
  engine.registerDecimalInputSymbol("goal.angle_to_center_of_free_part", &getCenterOfFreePart);
  // "goal.time_since_last_seen"
  engine.registerDecimalInputSymbol("goal.time_since_last_seen", &timeSinceGoalWasSeen);
  // "goal.was_seen"
  engine.registerBooleanInputSymbol("goal.was_seen", &goalWasSeen);
  // "goal.free_part_in_front_of_me"
  engine.registerBooleanInputSymbol("goal.free_part_in_front_of_me", &goalInFrontOfMe);
  // "goal.free_part_in_front_of_me_safe"
  engine.registerBooleanInputSymbol("goal.free_part_in_front_of_me_safe", &goalInFrontOfMeSafe);
  // "goal.free_part_size"
  engine.registerDecimalInputSymbol("goal.free_part_size", &getFreePartSize);
}

void BH2009GoalSymbols::update()
{
  timeSinceGoalWasSeen = static_cast<double>(theInstance->frameInfo.getTimeSince(theInstance->goalModel.timeWhenLastSeen));
  goalWasSeen          = timeSinceGoalWasSeen < 500;
}

bool BH2009GoalSymbols::goalInFrontOfMe()
{
  return theInstance->goalModel.largestFreePart.isInside(0.0);
}

bool BH2009GoalSymbols::goalInFrontOfMeSafe()
{
  const double safetyDist = 0.34906585039886590 / 2;
  const Range<double>& largestFreePart = theInstance->goalModel.largestFreePart;
  return largestFreePart.max > safetyDist && 
         largestFreePart.min < -safetyDist;
}

double BH2009GoalSymbols::getCenterOfFreePart()
{
  return toDegrees(theInstance->goalModel.largestFreePart.getCenter());
}

double BH2009GoalSymbols::getAngleToLeftEnd()
{
  return toDegrees(theInstance->goalModel.largestFreePart.max);
}

double BH2009GoalSymbols::getAngleToRightEnd()
{
  return toDegrees(theInstance->goalModel.largestFreePart.min);
}

double BH2009GoalSymbols::getFreePartSize()
{
  const Range<double>& largestFreePart = theInstance->goalModel.largestFreePart;
  return toDegrees(largestFreePart.max-largestFreePart.min);
}
