/** 
* @file BH2009HeadSymbols.h
*
* Declaration of class BH2009HeadSymbols.
*
* @author Max Risler
*/

#ifndef __BH2009HeadSymbols_h_
#define __BH2009HeadSymbols_h_

#include "../../Symbols.h"
#include "Representations/Infrastructure/CameraInfo.h"
#include "Representations/Perception/CameraMatrix.h"
#include "Representations/MotionControl/HeadMotionRequest.h"
#include "Representations/MotionControl/MotionInfo.h"
#include "Representations/Infrastructure/JointData.h"
#include "Representations/Modeling/RobotPose.h"
#include "Tools/Math/Vector3.h"

/**
* The Xabsl symbols that are defined in "head_symbols.xabsl"
*
* @author Max Risler
*/ 
class BH2009HeadSymbols : public Symbols
{
public:
  /*
  * Constructor.
  * @param headMotionRequest A reference to the HeadMotionRequest.
  * @param sensorData A reference to the SensorData.
  * @param cameraInfo A reference to the CameraInfo.
  * @param cameraMatrix A reference to the CameraMatrix.
  */
  BH2009HeadSymbols(HeadMotionRequest& headMotionRequest, SafeJointData const& jointData, CameraInfo const& cameraInfo, CameraMatrix const& cameraMatrix, RobotPose const& robotPose) : 
    headMotionRequest(headMotionRequest),
    jointData(jointData),
    cameraInfo(cameraInfo),
    cameraMatrix(cameraMatrix),
    robotPose(robotPose),
    lookAtPointX(0),
    lookAtPointY(0),
    offsetInImageX(0),
    offsetInImageY(0),
    lookAtPointHeadPan(0),
    lookAtPointHeadTilt(0)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC BH2009HeadSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** A reference to the HeadMotionRequest */
  HeadMotionRequest& headMotionRequest;

  /** A reference to the JointData */
  SafeJointData const& jointData;

  /** A reference to the CameraInfo */
  CameraInfo const& cameraInfo;

  /** A reference to the CameraMatrix */
  CameraMatrix const& cameraMatrix;

  RobotPose const& robotPose;

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  static void setTilt(double rot);
  static double getTilt();

  static double getExecutedTilt();

  static void setPan(double rot);
  static double getPan();

  static double getExecutedPan();

  static void setSpeed(double rot);
  static double getSpeed();

  static bool getPositionReached();

  double lookAtPointX;
  double lookAtPointY;
  double lookAtPointZ;

  double offsetInImageX;
  double offsetInImageY;
  
  double lookAtPointHeadPan;
  double lookAtPointHeadTilt;

  static void lookAtPointParametersChanged();

  virtual void update();

private:
  /**
  * Calculate angles to look at a given point.
  * Calculation taken from GT2006 HeadControl. */
  void calculateLookAtPoint();
};


#endif // __HeadSymbols_h_

