/** 
* @file BH2009LEDSymbols.h
*
* Declaration of class BH2009LEDSymbols.
*
* @author Judith Mller
* @author Andr Schreck
*/

#ifndef __BH2009LEDSymbols_h_
#define __BH2009LEDSymbols_h_

#include "../../Symbols.h"
#include "Representations/Infrastructure/LEDRequest.h"

/**
* The Xabsl symbols that are defined in "led_symbols.xabsl"
*
* @author Judith Mller
*/ 
class BH2009LEDSymbols : public Symbols
{
public:
  /*
  * Constructor.
  * @param ledRequest A reference to the LEDRequest.
  */
  BH2009LEDSymbols(LEDRequest& ledRequest) : 
    ledRequest(ledRequest)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC BH2009LEDSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** A reference to the LEDRequest */
  LEDRequest& ledRequest;

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

private:
  static void setFaceLeftRed(int ledState);
  static int getFaceLeftRed() { return (int)theInstance->ledRequest.ledStates[LEDRequest::faceLeftRed0Deg]; }

  static void setFaceLeftGreen(int ledState);
  static int getFaceLeftGreen() { return (int)theInstance->ledRequest.ledStates[LEDRequest::faceLeftGreen0Deg]; }

  static void setFaceLeftBlue(int ledState);
  static int getFaceLeftBlue() { return (int)theInstance->ledRequest.ledStates[LEDRequest::faceLeftBlue0Deg]; }

  static void setFaceRightRed(int ledState);
  static int getFaceRightRed() { return (int)theInstance->ledRequest.ledStates[LEDRequest::faceRightRed0Deg]; }

  static void setFaceRightGreen(int ledState);
  static int getFaceRightGreen() { return (int)theInstance->ledRequest.ledStates[LEDRequest::faceRightGreen0Deg]; }

  static void setFaceRightBlue(int ledState);
  static int getFaceRightBlue() { return (int)theInstance->ledRequest.ledStates[LEDRequest::faceRightBlue0Deg]; }
};


#endif // __LEDSymbols_h_
