/** 
* \file BH2009RoleSymbols.cpp
* Collection of symbols for dynamic role choosing.
* \author Colin Graf
*/

#include "BH2009RoleSymbols.h"
#include "BH2009TeamSymbols.h"
#include "BH2009GoalSymbols.h"
#include "BH2009SoccerSymbols.h"
#include "Tools/Team.h"


PROCESS_WIDE_STORAGE BH2009RoleSymbols* BH2009RoleSymbols::theInstance = 0;

void BH2009RoleSymbols::registerSymbols(xabsl::Engine& engine)
{
  engine.registerBooleanInputSymbol("role.designated_as_keeper", &designatedAsKeeper);
  engine.registerBooleanInputSymbol("role.nearest_to_ball", &BH2009SoccerSymbols::staticGetNearestToBall);
  engine.registerBooleanInputSymbol("role.second_nearest_to_ball", &BH2009SoccerSymbols::staticGetSecondNearestToBall);
  engine.registerBooleanInputSymbol("role.can_strike", &BH2009GoalSymbols::goalInFrontOfMe);
  engine.registerBooleanInputSymbol("role.ball.in_opponent_area", &BH2009SoccerSymbols::staticGetBallInOpponentArea);
  engine.registerDecimalInputSymbol("role.connected_players", &BH2009TeamSymbols::staticGetConnectedPlayers);
  engine.registerBooleanInputSymbol("role.lost_connection", &BH2009TeamSymbols::staticGetLostConnection);
  engine.registerBooleanInputSymbol("role.ball.hold", BH2009SoccerSymbols::staticGetBallHold);
  engine.registerEnumElement("role.robot_model", "role.robot_model.old_bioloid", RobotInfo::oldBioloid);
  engine.registerEnumElement("role.robot_model", "role.robot_model.new_bioloid", RobotInfo::newBioloid);
  engine.registerEnumElement("role.robot_model", "role.robot_model.nao", RobotInfo::nao);
  engine.registerEnumeratedInputSymbol("role.robot_model", "role.robot_model", &staticGetRobotModel);
  engine.registerBooleanOutputSymbol("role.is_penalty_striker", &isPenaltyStriker);
  engine.registerBooleanOutputSymbol("role.is_penalty_keeper", &isPenaltyKeeper);

  engine.registerEnumElement("game.role", "game.role.undefined", BehaviorData::undefined); 
  engine.registerEnumElement("game.role", "game.role.keeper", BehaviorData::keeper);
  engine.registerEnumElement("game.role", "game.role.defender", BehaviorData::defender);
  engine.registerEnumElement("game.role", "game.role.supporter", BehaviorData::supporter);
  engine.registerEnumElement("game.role", "game.role.striker", BehaviorData::striker);

  engine.registerEnumeratedOutputSymbol("game.role", "game.role", (int*)&behaviorData.role);

  engine.registerEnumElement("role.role", "role.role.undefined", BehaviorData::undefined); 
  engine.registerEnumElement("role.role", "role.role.keeper", BehaviorData::keeper);
  engine.registerEnumElement("role.role", "role.role.defender", BehaviorData::defender);
  engine.registerEnumElement("role.role", "role.role.supporter", BehaviorData::supporter);
  engine.registerEnumElement("role.role", "role.role.striker", BehaviorData::striker);

  engine.registerEnumeratedOutputSymbol("role.role", "role.role", (int*)&behaviorData.role);
  engine.registerEnumeratedInputSymbol("role.native_role", "role.role", (int*)&nativeRole);
}

void BH2009RoleSymbols::init() 
{
  if(robotInfo.number >= BehaviorData::firstRole && robotInfo.number < BehaviorData::numOfRoles)
    behaviorData.role = nativeRole = (BehaviorData::Role)robotInfo.number;
}

void BH2009RoleSymbols::update()
{
  designatedAsKeeper = robotInfo.number == 1;
}

int BH2009RoleSymbols::getRobotModel()
{
  return robotInfo.robotModel;
}
