/** 
* \file BH2009TeamSymbols.cpp
* Implementation of symbols for TeamMateData.
* \author Colin Graf
*/

#include "BH2009TeamSymbols.h"
#include "BH2009BallSymbols.h"
#include "BH2009LocatorSymbols.h"


PROCESS_WIDE_STORAGE BH2009TeamSymbols* BH2009TeamSymbols::theInstance = 0;

void BH2009TeamSymbols::registerSymbols(xabsl::Engine& engine)
{
  // team mate enumeration
  engine.registerEnumElement("team.mate", "team.mate.player1", 1);
  engine.registerEnumElement("team.mate", "team.mate.keeper", 1); 
  engine.registerEnumElement("team.mate", "team.mate.player2", 2);
  engine.registerEnumElement("team.mate", "team.mate.player3", 3);
  engine.registerEnumElement("team.mate", "team.mate.player4", 4);
  
  // team ball symbols
  engine.registerDecimalInputSymbol("team.ball.distance", &staticGetBallDistance);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.ball.distance", "team.ball.distance.player", "team.mate", &player);
  
  engine.registerDecimalInputSymbol("team.ball.angle", &staticGetBallAngle);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.ball.angle", "team.ball.angle.player", "team.mate", &player);
  
  engine.registerDecimalInputSymbol("team.ball.x", &staticGetBallX);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.ball.x", "team.ball.x.player", "team.mate", &player);
  
  engine.registerDecimalInputSymbol("team.ball.y", &staticGetBallY);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.ball.y", "team.ball.y.player", "team.mate", &player);

  engine.registerDecimalInputSymbol("team.ball.position.field.x", &staticGetBallPositionFieldX);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.ball.position.field.x", "team.ball.position.field.x.player", "team.mate", &player);
  
  engine.registerDecimalInputSymbol("team.ball.position.field.y", &staticGetBallPositionFieldY);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.ball.position.field.y", "team.ball.position.field.y.player", "team.mate", &player);

  // team locator symbols
  engine.registerDecimalInputSymbol("team.locator.pose.x", &staticGetLocatorPoseX);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.locator.pose.x", "team.locator.pose.x.player", "team.mate", &player);
  
  engine.registerDecimalInputSymbol("team.locator.pose.y", &staticGetLocatorPoseY);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.locator.pose.y", "team.locator.pose.y.player", "team.mate", &player);
  
  engine.registerDecimalInputSymbol("team.locator.pose.angle", &staticGetLocatorPoseAngle);
  engine.registerDecimalInputSymbolEnumeratedParameter("team.locator.pose.angle", "team.locator.pose.angle.player", "team.mate", &player);
}

double BH2009TeamSymbols::getBallDistance()
{
  if(player == robotInfo.number)
    return BH2009BallSymbols::staticGetBallDistance();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.ballModels[player].lastSeenEstimate.getDistance();
  return 0;
}

double BH2009TeamSymbols::getBallAngle()
{
  if(player == robotInfo.number)
    return BH2009BallSymbols::staticGetBallAngle();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return toDegrees(teamMateData.ballModels[player].lastSeenEstimate.getAngle());
  return 0;
}

double BH2009TeamSymbols::getBallX()
{
  if(player == robotInfo.number)
    return BH2009BallSymbols::staticGetBallPositionRobotX();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.ballModels[player].lastSeenEstimate.getPositionInRelativeCoordinates().x;
  return 0;
}

double BH2009TeamSymbols::getBallY()
{
  if(player == robotInfo.number)
    return BH2009BallSymbols::staticGetBallPositionRobotY();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.ballModels[player].lastSeenEstimate.getPositionInRelativeCoordinates().y;
  return 0;
}

double BH2009TeamSymbols::getBallPositionFieldX()
{
  if(player == robotInfo.number)
    return BH2009BallSymbols::staticGetBallPositionRobotX();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.ballModels[player].lastSeenEstimate.getPositionInFieldCoordinates(teamMateData.robotPoses[player]).x;
  return 0;
}

double BH2009TeamSymbols::getBallPositionFieldY()
{
  if(player == robotInfo.number)
    return BH2009BallSymbols::staticGetBallPositionRobotY();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.ballModels[player].lastSeenEstimate.getPositionInFieldCoordinates(teamMateData.robotPoses[player]).y;
  return 0;
}

double BH2009TeamSymbols::getLocatorPoseX()
{
  if(player == robotInfo.number)
    return BH2009LocatorSymbols::getPoseX();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.robotPoses[player].translation.x;
  return 0;
}

double BH2009TeamSymbols::getLocatorPoseY()
{
  if(player == robotInfo.number)
    return BH2009LocatorSymbols::getPoseY();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return teamMateData.robotPoses[player].translation.y;
  return 0;
}

double BH2009TeamSymbols::getLocatorPoseAngle()
{
  if(player == robotInfo.number)
    return BH2009LocatorSymbols::getPoseAngle();
  if(player >= TeamMateData::firstPlayer && player < TeamMateData::numOfPlayers)
    return toDegrees(teamMateData.robotPoses[player].getAngle());
  return 0;
}

double BH2009TeamSymbols::getConnectedPlayers()
{
  return teamMateData.numOfConnectedPlayers;
}

bool BH2009TeamSymbols::getLostConnection()
{
  return teamMateData.numOfConnectedPlayers <= 0 && teamMateData.wasConnected;
}
