/** 
* \file BH2009TeamSymbols.h
* Implementation of symbols for TeamMateData.
* \author Colin Graf
*/

#ifndef __BH2009TeamSymbols_h_
#define __BH2009TeamSymbols_h_

#include "../../Symbols.h"
#include "Representations/Infrastructure/RobotInfo.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/Infrastructure/TeamMateData.h"
#include "Representations/Modeling/BallModel.h"
#include "Representations/Configuration/FieldDimensions.h"

/**
* \class BH2009TeamSymbols
* The Xabsl symbols that are defined in "team_symbols.xabsl"
* \author Colin Graf
*/ 
class BH2009TeamSymbols : public Symbols
{
public:
  /**
  * Constructor.
  * \param robotInfo A reference to the RobotInfo.
  * \param robotPose A reference to the RobotPose.
  * \param ballModel A reference to the BallModel.
  * \param teamMateData A refernece to the TeamMateData.
  * \param fieldDimensions A reference to the FieldDimensions.
  * \param frameInfo a reference to the FrameInfo.
  */
  BH2009TeamSymbols(
    const RobotInfo& robotInfo, 
    const RobotPose& robotPose,
    const BallModel& ballModel,
    const TeamMateData& teamMateData,
    const FieldDimensions& fieldDimensions,
    const FrameInfo& frameInfo) : 
      robotInfo(robotInfo),
      ballModel(ballModel),
      robotPose(robotPose),
      teamMateData(teamMateData),
      fieldDimensions(fieldDimensions),
      frameInfo(frameInfo)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC BH2009TeamSymbols* theInstance;

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  /** updates the symbols */
  void update() {};

private:

  double getBallDistance();
  double getBallAngle();
  double getBallX();
  double getBallY();
  double getBallPositionFieldX();
  double getBallPositionFieldY();
  double getLocatorPoseX();
  double getLocatorPoseY();
  double getLocatorPoseAngle();
  double getConnectedPlayers();
  bool getLostConnection();

  // data sources
  const RobotInfo& robotInfo;
  const BallModel& ballModel;
  const RobotPose& robotPose;
  const TeamMateData& teamMateData;
  const FieldDimensions& fieldDimensions;
  const FrameInfo& frameInfo;

  // symbol parameter
  int player;

  // static symbol provider functions for the xabsl engine
  static double staticGetBallDistance() { return theInstance->getBallDistance(); };
  static double staticGetBallAngle() { return theInstance->getBallAngle(); };
  static double staticGetBallX() { return theInstance->getBallX(); };
  static double staticGetBallY() { return theInstance->getBallY(); };
  static double staticGetBallPositionFieldX() { return theInstance->getBallPositionFieldX(); };
  static double staticGetBallPositionFieldY() { return theInstance->getBallPositionFieldY(); };
  static double staticGetLocatorPoseX() { return theInstance->getLocatorPoseX(); };
  static double staticGetLocatorPoseY() { return theInstance->getLocatorPoseY(); };
  static double staticGetLocatorPoseAngle() { return theInstance->getLocatorPoseAngle(); };
  static double staticGetConnectedPlayers() { return theInstance->getConnectedPlayers(); }
  static bool staticGetLostConnection() { return theInstance->getLostConnection(); }

  friend class BH2009SoccerSymbols; 
  friend class BH2009RoleSymbols;
};


#endif // __BH2009TeamSymbols_h_

