/** 
* @file MotionSymbols.cpp
*
* Implementation of class MotionSymbols.
*
* @author Max Risler
*/

#include "MotionSymbols.h"
#include "Tools/Xabsl/GT/GTXabslTools.h"

PROCESS_WIDE_STORAGE MotionSymbols* MotionSymbols::theInstance = 0;

void MotionSymbols::registerSymbols(xabsl::Engine& engine)
{
  char s[256];
  int i;
  
  // "motion.type"
  for (i=0;i<MotionRequest::numOfMotions;i++)
  {
    sprintf(s, "motion.type.");
    getXabslString(s + strlen(s),MotionRequest::getMotionName((MotionRequest::Motion)i));
    engine.registerEnumElement("motion.type",s,i);
  }
  engine.registerEnumeratedOutputSymbol("motion.type","motion.type", (int*)&motionRequest.motion);
  
  // "executed_motion.type"
  engine.registerEnumeratedInputSymbol("executed_motion.type","motion.type", (const int*)&motionInfo.executedMotionRequest.motion);

  // "motion.walk_speed.x"
  engine.registerDecimalOutputSymbol("motion.walk_speed.x",
    &motionRequest.walkRequest.speed.translation.x);
  // "executed_motion.walk_speed.x"
  engine.registerDecimalInputSymbol("executed_motion.walk_speed.x",
    &motionInfo.executedMotionRequest.walkRequest.speed.translation.x);

  // "motion.walk_speed.y"
  engine.registerDecimalOutputSymbol("motion.walk_speed.y",
    &motionRequest.walkRequest.speed.translation.y);
  // "executed_motion.walk_speed.y"
  engine.registerDecimalInputSymbol("executed_motion.walk_speed.y",
    &motionInfo.executedMotionRequest.walkRequest.speed.translation.y);
  
  // "motion.walk_speed.rot"
  engine.registerDecimalOutputSymbol("motion.walk_speed.rot",
    &setWalkSpeedRot,
    &getWalkSpeedRot);
  // "executed_motion.walk_speed.rot"
  engine.registerDecimalInputSymbol("executed_motion.walk_speed.rot",
    &getExecutedWalkSpeedRot);

  // "motion.special_action"
  for (i=0;i<SpecialActionRequest::numOfSpecialActions;i++)
  {
    sprintf(s, "motion.special_action.");
    getXabslString(s + strlen(s),SpecialActionRequest::getSpecialActionName((SpecialActionRequest::SpecialActionID)i));
    engine.registerEnumElement("motion.special_action",s,i);
  }
  engine.registerEnumeratedOutputSymbol("motion.special_action","motion.special_action", (int*)&motionRequest.specialActionRequest.specialAction);

  engine.registerEnumeratedOutputSymbol("motion.last_special_action","motion.special_action", &lastSpecialAction);

  // "executed_motion.special_action"
  engine.registerEnumeratedInputSymbol("executed_motion.special_action","motion.special_action", (const int*)&motionInfo.executedMotionRequest.specialActionRequest.specialAction);

  // "motion.special_action.mirror"
  engine.registerBooleanOutputSymbol("motion.special_action.mirror", &motionRequest.specialActionRequest.mirror);
  // "executed_motion.special_action.mirror"
  engine.registerBooleanInputSymbol("executed_motion.special_action.mirror", &motionInfo.executedMotionRequest.specialActionRequest.mirror);
}


void MotionSymbols::setWalkSpeedRot(double rot)
{
  theInstance->motionRequest.walkRequest.speed.rotation = fromDegrees(rot);
}

double MotionSymbols::getWalkSpeedRot()
{
  return toDegrees(theInstance->motionRequest.walkRequest.speed.rotation);
}

double MotionSymbols::getExecutedWalkSpeedRot()
{
  return toDegrees(theInstance->motionInfo.executedMotionRequest.walkRequest.speed.rotation);
}
