/** 
* @file MotionSymbols.h
*
* Declaration of class MotionSymbols.
*
* @author Max Risler
*/

#ifndef __MotionSymbols_h_
#define __MotionSymbols_h_

#include "../Symbols.h"
#include "Representations/MotionControl/MotionRequest.h"
#include "Representations/MotionControl/MotionInfo.h"

/**
* The Xabsl symbols that are defined in "motion_symbols.xabsl"
*
* @author Max Risler
*/ 
class MotionSymbols : public Symbols
{
public:
  /*
  * Constructor.
  * @param motionRequest A reference to the MotionRequest.
  * @param motionInfo A reference to the MotionInfo.
  */
  MotionSymbols(MotionRequest& motionRequest, MotionInfo const& motionInfo) : 
    motionRequest(motionRequest),
    motionInfo(motionInfo),
    lastSpecialAction(0)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC MotionSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** A reference to the MotionRequest */
  MotionRequest& motionRequest;

  /** A reference to the MotionInfo */
  MotionInfo const& motionInfo;

  int lastSpecialAction;

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  static void setWalkSpeedRot(double rot);
  static double getWalkSpeedRot();
  static double getExecutedWalkSpeedRot();
};


#endif // __MotionSymbols_h_

