/** 
* @file CognitionLogDataProvider.h
* This file declares a module that provides data replayed from a log file.
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#ifndef __CognitionLogDataProvider_h_
#define __CognitionLogDataProvider_h_

#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/Image.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/Perception/PointsPercept.h"
#include "Representations/Perception/BallPercept.h"
#include "Representations/Modeling/RobotPose.h"
#include "Representations/Modeling/BallModel.h"
#include "Tools/MessageQueue/InMessage.h"

MODULE(CognitionLogDataProvider)
  PROVIDES_WITH_OUTPUT(Image)
  PROVIDES(CameraInfo)
  PROVIDES_WITH_MODIFY(FrameInfo)
  PROVIDES_WITH_MODIFY_AND_OUTPUT_AND_DRAW(PointsPercept)
  PROVIDES_WITH_MODIFY_AND_OUTPUT_AND_DRAW(BallPercept)
  PROVIDES_WITH_MODIFY_AND_DRAW(RobotPose)
  PROVIDES_WITH_MODIFY_AND_DRAW(BallModel)
  PROVIDES_WITH_MODIFY_AND_OUTPUT_AND_DRAW(GroundTruthRobotPose)
  PROVIDES_WITH_MODIFY_AND_OUTPUT_AND_DRAW(GroundTruthBallModel)
END_MODULE

class CognitionLogDataProvider : public CognitionLogDataProviderBase
{
private:
  PROCESS_WIDE_STORAGE_STATIC CognitionLogDataProvider* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */
  bool frameDataComplete; /**< Were all messages of the current frame received? */

  Image* theImage;
  CameraInfo* theCameraInfo;
  FrameInfo* theFrameInfo;
  PointsPercept* thePointsPercept;
  BallPercept* theBallPercept;
  BallModel* theBallModel;
  RobotPose* theRobotPose;
  GroundTruthBallModel* theGroundTruthBallModel;
  GroundTruthRobotPose* theGroundTruthRobotPose;
  
  void update(Image& image);
  void update(CameraInfo& cameraInfo);
  void update(FrameInfo& frameInfo);
  void update(PointsPercept& pointsPercept);
  void update(BallPercept& ballPercept);
  void update(BallModel& ballModel);
  void update(RobotPose& robotPose);
  void update(GroundTruthBallModel& groundTruthBallModel);
  void update(GroundTruthRobotPose& groundTruthRobotPose);

  /** 
  * The method is called for every incoming debug message by handleMessage.
  * @param message An interface to read the message from the queue.
  * @return Was the message handled?
  */
  bool handleMessage2(InMessage& message);

public:
  /**
  * Default constructor.
  */
  CognitionLogDataProvider();

  /**
  * Destructor.
  */
  ~CognitionLogDataProvider();

  /** 
  * The method is called for every incoming debug message.
  * @param message An interface to read the message from the queue.
  * @return Was the message handled?
  */
  static bool handleMessage(InMessage& message);

  /** 
  * The method returns whether idProcessFinished was received.
  * @return Were all messages of the current frame received?
  */
  static bool isFrameDataComplete();
};

#endif // __CognitionLogDataProvider_h_
