/**
* @file Modules/Infrastructure/MotionRobotHealthProvider.h
* This file implements a module that provides information about the robot's health.
* @author <a href="mailto:timlaue@informatik.uni-bremen.de">Tim Laue</a>
*/

#include "MotionRobotHealthProvider.h"


void MotionRobotHealthProvider::update(MotionRobotHealth& motionRobotHealth)
{
  // Compute frame rate of motion process:
  timeBuffer.add(SystemCall::getCurrentSystemTime());
  unsigned timeDiffSum(0);
  int numOfTimeDiffs(0);
  for(int i=0; i < timeBuffer.getNumberOfEntries() - 1; ++i)
  {
    timeDiffSum += (timeBuffer[i] - timeBuffer[i+1]);
    ++numOfTimeDiffs;
  }
  motionRobotHealth.motionFrameRate = timeDiffSum ? 1000.0F / (static_cast<float>(timeDiffSum)/numOfTimeDiffs) : 0.0F;

  // Directly transfer temperature and voltage data from SensorData:
  motionRobotHealth.voltage = theSafeSensorData.data[SensorData::voltage];
  unsigned char maxTemperature(0);
  for(int i=0; i<JointData::numOfJoints; ++i)
    if(theSafeSensorData.temperatures[i] > maxTemperature)
      maxTemperature = theSafeSensorData.temperatures[i];
  motionRobotHealth.maxTemperature = static_cast<float>(maxTemperature);

  // Compute the number of frames from robot which have been received within the last second:
  if(sensorTimeBuffer.getNumberOfEntries() == 0 && theSafeSensorData.timeStamp != 0)
    sensorTimeBuffer.add(theSafeSensorData.timeStamp);
  else if(theSafeSensorData.timeStamp != sensorTimeBuffer[0])
    sensorTimeBuffer.add(theSafeSensorData.timeStamp);
  if(sensorTimeBuffer.getNumberOfEntries())
  {
    unsigned timeDiff = SystemCall::getCurrentSystemTime() - sensorTimeBuffer[sensorTimeBuffer.getNumberOfEntries()-1];
    motionRobotHealth.robotFramesPerSecond = static_cast<float>(sensorTimeBuffer.getNumberOfEntries()) / timeDiff;
    motionRobotHealth.robotFramesPerSecond *= 1000;
  }
  else
    motionRobotHealth.robotFramesPerSecond = 0.0F;
}

MAKE_MODULE(MotionRobotHealthProvider, Infrastructure)
