/** 
* @file OracleDataProvider.h
* This file declares a module that provides the data received from the simulator oracle.
* @author Max Risler
*/

#ifndef __OracleDataProvider_h_
#define __OracleDataProvider_h_

#include "Platform/Oracle.h"
#include "Tools/Module/Module.h"
#include "Representations/Modeling/RobotPose.h"
#include "Representations/Modeling/BallModel.h"

MODULE(OracleDataProvider)
  PROVIDES_WITH_MODIFY_AND_DRAW(RobotPose)
  PROVIDES_WITH_MODIFY_AND_DRAW(BallModel)
  PROVIDES_WITH_MODIFY_AND_DRAW(GroundTruthRobotPose)
  PROVIDES_WITH_MODIFY_AND_DRAW(GroundTruthBallModel)
END_MODULE

class OracleDataProvider : public OracleDataProviderBase
{
private:
  void update(RobotPose& robotPose);
  void update(BallModel& ballModel);
  void update(GroundTruthRobotPose& groundTruthRobotPose);
  void update(GroundTruthBallModel& groundTruthBallModel);

  PROCESS_WIDE_STORAGE_STATIC Oracle* oracle; /**< The oracle */

  friend class Cognition;
};

#endif // __OracleDataProvider_h_
