/**
* @file Modules/Infrastructure/RobotHealthProvider.h
* This file implements a module that provides information about the robot's health.
* @author <a href="mailto:timlaue@informatik.uni-bremen.de">Tim Laue</a>
*/

#include "RobotHealthProvider.h"
#include "Tools/Settings.h"
#include "Tools/Team.h"
#include "Tools/Debugging/ReleaseOptions.h"


void RobotHealthProvider::update(RobotHealth& robotHealth)
{
  // Transfer information from other process:
  robotHealth = theMotionRobotHealth;

  // Add memory load and robot name:
  robotHealth.memoryLoad = static_cast<unsigned char>(SystemCall::getMemoryLoad());
  robotHealth.robotName = Global::getSettings().robot;
  
  // Compute frame rate of cognition process:
  timeBuffer.add(SystemCall::getCurrentSystemTime());
  unsigned timeDiffSum(0);
  int numOfTimeDiffs(0);
  for(int i=0; i < timeBuffer.getNumberOfEntries() - 1; ++i)
  {
    timeDiffSum += (timeBuffer[i] - timeBuffer[i+1]);
    ++numOfTimeDiffs;
  }
  robotHealth.cognitionFrameRate = timeDiffSum ? 1000.0F / (static_cast<float>(timeDiffSum)/numOfTimeDiffs) : 0.0F;

  // Communicate health:
  if(Global::getReleaseOptions().robotHealth)
  {
    TEAM_OUTPUT(idRobotHealth, bin, robotHealth);
  }
}

MAKE_MODULE(RobotHealthProvider, Infrastructure)
