/**
* @file Modules/Infrastructure/RobotHealthProvider.h
* This file declares a module that provides information about the robot's health.
* @author <a href="mailto:timlaue@informatik.uni-bremen.de">Tim Laue</a>
*/

#ifndef __RobotHealth_H__
#define __RobotHealth_H__

#include "Tools/Module/Module.h"
#include "Tools/RingBuffer.h"
#include "Representations/Infrastructure/RobotHealth.h"

MODULE(RobotHealthProvider)
  REQUIRES(MotionRobotHealth)
  PROVIDES(RobotHealth)
END_MODULE


/**
* @class RobotHealthProvider
* A module that provides information about the robot's health
*/
class RobotHealthProvider : public RobotHealthProviderBase
{
private:
  /** The main function, called every cycle
  * @param robotHealth The data struct to be filled
  */
  void update(RobotHealth& robotHealth);

  /** Buffered timestamps of previous executions */
  RingBuffer<unsigned, 30> timeBuffer;    

public:
  /** Constructor. */
  RobotHealthProvider() {}
};

#endif  //__RobotHealthProvider_h_
