/**
* @file Modules/Infrastructure/SensorSAP.h
* The file declares a module that is useful to calibrate the sensors.
* @author <a href="mailto:judy@tzi.de">Judith Mller</a>
*/

#include "SensorSAP.h"
#include "Tools/Debugging/DebugDrawings.h"


void SensorSAP::update(SensorSAPOutput&)
{
  //Buffer data:
  buffers[accX].add(theSafeSensorData.data[SensorData::accX]);
  buffers[accY].add(theSafeSensorData.data[SensorData::accY]);
  buffers[accZ].add(theSafeSensorData.data[SensorData::accZ]);
  buffers[gyroX].add(theSafeSensorData.data[SensorData::gyroX]);
  buffers[gyroY].add(theSafeSensorData.data[SensorData::gyroY]);
  buffers[gyroZ].add(theSafeSensorData.data[SensorData::gyroZ]);
  buffersC[compassX].add(theSafeSensorData.data[SensorData::compassX]);
  buffersC[compassY].add(theSafeSensorData.data[SensorData::compassY]);

  PLOT("module:SensorSAP:compassX", buffersC[compassX].getAverage());
  PLOT("module:SensorSAP:compassY", buffersC[compassY].getAverage());
  PLOT("module:SensorSAP:compassYXAtan2inDegree", 
    (double)(toDegrees(atan2(buffersC[compassX].getAverage() - theSensorCalibration.compassXOffset, 
    buffersC[compassY].getAverage() - theSensorCalibration.compassYOffset))));
  PLOT("module:SensorSAP:accX", buffers[accX].getAverage()*1000.0);
  PLOT("module:SensorSAP:accY", buffers[accY].getAverage()*1000.0);
  PLOT("module:SensorSAP:accZ", buffers[accZ].getAverage()*1000.0);
  PLOT("module:SensorSAP:gyroX", toDegrees(buffers[gyroX].getAverage()));
  PLOT("module:SensorSAP:gyroY", toDegrees(buffers[gyroY].getAverage()));
  PLOT("module:SensorSAP:gyroZ", buffers[gyroZ].getAverage());
}

MAKE_MODULE(SensorSAP, Infrastructure)
