/**
* @file Modules/Infrastructure/SensorSAP.h
* The file declares a module that is useful to calibrate the sensors.
* @author <a href="mailto:judy@tzi.de">Judith Müller</a>
*/

#ifndef __SensorSAP_H__
#define __SensorSAP_H__

#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/SensorData.h"
#include "Representations/Infrastructure/SensorSAPOutput.h"
#include "Representations/Configuration/SensorCalibration.h"
#include "Tools/RingBufferWithSum.h"
#include "Tools/Streams/Streamable.h"

MODULE(SensorSAP)
  REQUIRES(SafeSensorData)
  REQUIRES(SensorCalibration)
  PROVIDES(SensorSAPOutput)
END_MODULE


class SensorSAP : public SensorSAPBase
{
private:
  void update(SensorSAPOutput& sensorSAPOutput);

  /** Indices for buffers of compass data */
	enum BufferEntriesC{compassX = 0, compassY, numOfBuffersC};
   
  /** Indices for buffers of the other sensor data */
  enum BufferEntries{accX = 0, accY, accZ, gyroX, gyroY, gyroZ, numOfBuffers};
	
 /** Buffers for averaging compass data */
	RingBufferWithSum<double,5> buffersC[numOfBuffersC];

  /** Buffers for averaging the other sensor data */
  RingBufferWithSum<double,5> buffers[numOfBuffers];

public:
  /**
  * Constructor.
  */
  SensorSAP(){};
};


#endif  //__SensorSAP_h_
