/** 
* @file Modules/MotionControl/MotionSelector.h
* This file declares a module that is responsible for controlling the motion.
* @author <A href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</A>
*/

#ifndef __MotionSelector_h_
#define __MotionSelector_h_

#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/MotionControl/SpecialActionsOutput.h"
#include "Representations/MotionControl/WalkingEngineOutput.h"
#include "Representations/MotionControl/WalkingEngineStandOutput.h"
#include "Representations/MotionControl/MotionRequest.h"
#include "Representations/MotionControl/MotionSelection.h"

MODULE(MotionSelector)
  USES(SpecialActionsOutput)
  USES(WalkingEngineOutput)
  USES(WalkingEngineStandOutput)
  REQUIRES(FrameInfo)
  REQUIRES(RobotInfo)
  REQUIRES(MotionRequest)
  PROVIDES_WITH_MODIFY(MotionSelection)
END_MODULE

class MotionSelector : public MotionSelectorBase
{
private:
  MotionRequest::Motion lastMotion; /**< The motion requested in the previous frame. */
  MotionRequest::Motion prevMotion;
  unsigned switchTime; /**< The time when motions were switched, i.e start of interpolation. */
  int playDeadDelay; /**< Additional delay when interpolating from playDead. */

  void update(MotionSelection& motionSelection);

public:
  /**
  * Default constructor.
  */
  MotionSelector() : lastMotion(MotionRequest::specialAction), switchTime(0), playDeadDelay(2000) {}
};

#endif // __MotionSelector_h_
