/**
* @file NaoCameraMatrixProvider.h
* This file declares a class to calculate the position of the camera for the Nao.
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#ifndef __NaoCameraMatrixProvider_h_
#define __NaoCameraMatrixProvider_h_

#include "Tools/Module/Module.h"
#include "Tools/Debugging/DebugDrawings3D.h"
#include "Representations/Configuration/CameraCalibration.h"
#include "Representations/Configuration/RobotDimensions.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/Infrastructure/RobotInfo.h"
#include "Representations/Infrastructure/JointData.h"
#include "Representations/Perception/CameraMatrix.h"
#include "Representations/Modeling/FallDownState.h"
#include "Representations/MotionControl/MotionInfo.h"

MODULE(NaoCameraMatrixProvider)
  REQUIRES(FrameInfo)
  REQUIRES(RobotInfo)
  REQUIRES(CameraCalibration)
  REQUIRES(RobotDimensions)
  REQUIRES(SafeJointData)
  REQUIRES(MotionInfo)
  REQUIRES(FallDownState)
  PROVIDES_WITH_MODIFY_AND_DRAW(CameraMatrix);
END_MODULE

class NaoCameraMatrixProvider: public NaoCameraMatrixProviderBase
{
private:
  void update(CameraMatrix& cameraMatrix);

  void lowerFootPositionFromJoints(bool isLeft, double j0, double j1, double j2, double j3, 
                                   double j4, double j5, Pose3D& lowerFootPosition) const;
};

#endif// __NaoCameraMatrixProvider_h_
