/** 
* @file Platform/SimRobotQt/LocalRobot.h
*
* Declaration of LocalRobot.
*
* @author <A href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</A>
*/


#ifndef __LocalRobot_H__
#define __LocalRobot_H__

#include <ControllerQt/RobotConsole.h>
#include "Representations/Configuration/JointCalibration.h"
#include "Representations/Infrastructure/JointData.h"
#include "Representations/Infrastructure/LEDRequest.h"
#include <SimRobotCore/Controller/Controller.h>

/**
* @class LocalRobot
*
* A process that is instantiated to either directly control a physical robot,
* a simulated one, or to replay a logfile.
*/
class LocalRobot : public RobotConsole
{
private:
  DEBUGGING;
  int spJoints[JointData::numOfJoints], /**< The handles to the sensor ports of the joints. */      
      apJoints[JointData::numOfJoints], /**< The handles to the actuator ports of the joints. */
      psJoints[JointData::numOfJoints]; /**< The handles to the power switches of the joints. */
  int spCamera; /**< The handle to the sensor port of the virtual camera. */
  int spAcc;    /**< The handle to the sensor port of the virtual accelerometer. */
  int spGyro;   /**< The handle to the sensor port of the virtual gyrosope. */
  Image image; /**< The simulated image sent to the robot code. */
  int apLeds[LEDRequest::numOfLEDs]; /**< The handles to the actuator ports of the LEDs. */
  JointData jointData; /**< The simulated joint measurements sent to the robot code. */
  SensorData sensorData; /**< The simulated sensor data sent to the robot code. */
  unsigned imageTimeStamp,
           imageLastTimeStampSent;
  SimObject* objRobot; /**< A pointer to the associated SimRobot robot. */
  SimObject* objBody; /**< A pointer to the associated SimRobot robot's body. */
  JointCalibration jointCalibration; /**< The simulated robot is perfectly calibrated, but we need the signs. */
  Settings::Model model; /**< The model type of the simulated robot. */

public:
  /**
  * Constructor.
  */
  LocalRobot();

  /**
  * The function is called from the framework once in every frame 
  */
  virtual int main();

  /**
  * The function must be called to exchange data with SimRobot.
  * It sends the motor commands to SimRobot and acquires new sensor data.
  */
  void update();
};

#endif // __LocalRobot_H__
