/**
 * @file Platform/SimRobotQt/Oracle.cpp
 * 
 * Implementation of class Oracle for Win32.
 */

#include <Simulation/SimObject.h>

#include "Oracle.h"
#include "RoboCupCtrl.h"
#include "../SystemCall.h"
#include "../../Tools/Math/Geometry.h"


Oracle::Oracle()
{
  ctrl = RoboCupCtrl::getController();
  std::string robotName = RoboCupCtrl::getRobotFullName();
  me = SystemCall::getMode() == SystemCall::simulatedRobot ? ctrl->getObjectReference(robotName) : 0;
}

bool Oracle::isBlue() const
{
  if (!me) return false;
  return me->getName()[5] < '5';
}

RobotPose Oracle::getRobotPose() const
{
  RobotPose rp;
  if (me)
  {
    rp = ctrl->getPose2D(me);
    rp.translation *= 1000;
    if(isBlue())
      rp = (Pose2D(double(pi)) + rp);
  }
  return rp;
}

BallModel Oracle::getBallModel() 
{
  BallModel bm;
  SimObject* balls = ctrl->getObjectReference("RoboCup.balls");
  if(balls && balls->getNumberOfChildNodes())
  {
    RobotPose rp(getRobotPose());
    bm.timeWhenLastSeen = SystemCall::getCurrentSystemTime();
    Pose2D pose = ctrl->getPose2D(balls->getChildNode(0));
    if(isBlue())
      pose = (Pose2D(pi) + pose);
    Vector2<double> velocity(pose.translation * 1000.0 - lastBallModel.estimate.getPositionInFieldCoordinates(rp));
    velocity /= ((bm.timeWhenLastSeen - lastBallModel.timeWhenLastSeen) / 1000.0);
    bm.lastPerception.setPositionAndVelocityInFieldCoordinates(pose.translation * 1000.0,
                                                               velocity, rp);
    bm.estimate = bm.lastPerception;
    bm.lastSeenEstimate = bm.lastPerception;
    lastBallModel = bm;
  }
  return bm;
}
