/** 
* @file  Platform/SimRobotQt/SystemCall.cpp
* @brief static class for system calls
* @attention this is the SimRobotQt implementation
*/

#include "SystemCall.h"
#include <ControllerQt/ConsoleRoboCupCtrl.h>
#include <sys/timeb.h>

unsigned SystemCall::getCurrentSystemTime() 
{
  if(RoboCupCtrl::getController())
    return RoboCupCtrl::getController()->getTime();
  else
    return getRealSystemTime();
}

unsigned SystemCall::getRealSystemTime() 
{/*
  if(RoboCupCtrl::getController())
    return RoboCupCtrl::getController()->getCurrentSystemTime();
  else
  { */
#ifdef _WIN32
    return timeGetTime();
#else
    static unsigned base = 0;
    timeb sysTime;
    ftime(&sysTime);
    unsigned time = unsigned(sysTime.time * 1000 + sysTime.millitm);
    if(!base)
      base = time - 10000; // avoid time == 0, because it is often used as a marker
    return time - base;
#endif
  /*} */
}

const char* SystemCall::getHostName()
{
  static char buf[100];
  VERIFY(!gethostname(buf, sizeof(buf)));
  return buf;
}

SystemCall::Mode SystemCall::getMode()
{
  return ConsoleRoboCupCtrl::getMode();
}

int SystemCall::getMemoryLoad()
{
#ifdef _WIN32
  MEMORYSTATUS memStat;
  memStat.dwLength = sizeof(MEMORYSTATUS);
  GlobalMemoryStatus(&memStat);
  return memStat.dwMemoryLoad;
#else
  return 0; //Not implemented yet
#endif
}

void SystemCall::sleep(unsigned int ms)
{
#ifdef _WIN32
  Sleep(ms);
#else
  usleep(ms * 1000);
#endif
}


