/**
* @file Platform/linux/File.cpp
* 
* Implementation of class File for Linux.
*/

#include "Platform/File.h"

#include <stdarg.h>
#include <string.h>
#include "Platform/GTAssert.h"

#include <stdlib.h> // for getenv
#include <sys/stat.h>
#include <iostream>
#include <string>

File::File(const std::string& name,const std::string& mode)
{
  std::string fileName;
  if(name[0] != '/' && name[0] != '\\' && name[0] != '.' && (name[0] == 0 || name[1] != ':'))
    fileName = std::string(getGTDir()) + "/Config/" + name;
  else
    fileName = name;
#ifdef TARGET_SIM
  for(int i = fileName.length(); i >= 0; i--)
    if(fileName[i] == '\\')
      fileName[i] = '/';
#endif
  stream = fopen(fileName.c_str(), mode.c_str());
}

File::~File()
{
  if(stream != 0)
    fclose(stream);
}

void File::expandFileName(char* expandedName, const char* originalName)
{
  if(originalName[0] != '/' && originalName[0] != '\\' && originalName[0] != '.' && (originalName[0] == 0 || originalName[1] != ':'))
    sprintf(expandedName,"%s/Config/",getGTDir());
 else
    expandedName[0] = 0;

  ASSERT(strlen(expandedName) + strlen(originalName) < FILENAME_MAX);
  strcat(expandedName,originalName);
}

void File::read(void* p,unsigned size)
{
  VERIFY(!eof());
  fread(p,1,size,stream);
}

void File::write(const void *p,unsigned size)
{
  //if opening failed, stream will be 0 and fwrite would crash
  ASSERT(stream!=0);
  VERIFY(fwrite(p,1,size,stream) == size);
}

void File::printf(const char* format, ...)
{
  va_list args;
  va_start(args,format);
  vfprintf(stream,format,args);
  va_end(args);
}

bool File::eof()
{
  //never use feof(stream), because it informs you only after reading
  //to far and is never reset, e.g. if the stream grows afterwards
  //our implementation can handle both correctly:
  if(! stream)
    return false;
  else
  {
    int c = fgetc(stream);
    if(c==EOF)
      return true;
    else
    {
      VERIFY(ungetc(c,stream)!=EOF);
      return false;
    }
  }
}

char* File::getGTDir()
{
  static char dir[FILENAME_MAX] = {0};
  if(!dir[0])
  {
    char* result = getenv("GT_DIR");
    if (result)
    {
      strcpy(dir, result); 
    }
    else
    {
#ifdef SIMROBOTQT
      getcwd(dir, sizeof(dir) - 7);
      char* end = dir + strlen(dir) - 1;
      struct stat buff;;
      for(;;)
      {
        if(*end == '/' || end == dir - 1)
        {
          strcpy(end + 1, "Config");
          if(stat(dir, &buff) == 0)
            if(S_ISDIR(buff.st_mode))
            {
              end[end > dir ? 0 : 1] = '\0';
              return dir;
            }
        }
        if(end < dir)
          break;
        end--;
      }
#endif
      strcpy(dir, "/opt/bhuman/");
    }
  }
  return dir;
}

void File::flush()
{
  fflush (stream);
}
