/** 
* @file Platform/linux/GameHandler.cpp
* The file declares a class to communicate with the GameController.
* @author <A href="mailto:Thomas.Roefer@dfki.de">Thomas Röfer</A>
*/

#include "GameHandler.h"
#include "Platform/GTAssert.h"
#include "Platform/SystemCall.h"
#include <cstring> // needed for memcmp

#include <sys/select.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>

#define NON_BLOCK(socket) fcntl(socket,F_SETFL,O_NONBLOCK)
#define CLOSE(socket) close(socket)

GameHandler::~GameHandler()
{
  CLOSE(udpSocket);
}

void GameHandler::start(int port)
{
  if(this->port)
    CLOSE(udpSocket);

  this->port = port;

  // Socket for receiving broadcast messages
  udpSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  ASSERT(udpSocket != -1);

  sockaddr_in address;
  address.sin_family = AF_INET;
  address.sin_addr.s_addr = INADDR_ANY;
  address.sin_port = htons(port);
  int broadcast = 1;
  setsockopt(udpSocket, SOL_SOCKET, SO_BROADCAST, (const char *) &broadcast, sizeof(broadcast));
  int reuse = 1;
  setsockopt(udpSocket, SOL_SOCKET, SO_REUSEADDR, (const char*) &reuse, sizeof(reuse));
  NON_BLOCK(udpSocket);
  bind(udpSocket, (struct sockaddr *) &address, sizeof(address)); 
}

void GameHandler::send()
{
  // implement manual unpenalize package here
}

bool GameHandler::receive(RoboCup::RoboCupGameControlData& data)
{
  if(!port)
    return false; // not started yet

  bool received = false;
  int size;
  do
  {
    RoboCup::RoboCupGameControlData buffer;
    size = recv(udpSocket, (char*)&buffer, sizeof(buffer), 0);
    if(size == sizeof(buffer) && !std::memcmp(&buffer, GAMECONTROLLER_STRUCT_HEADER, 4) &&
       buffer.version == GAMECONTROLLER_STRUCT_VERSION)
    {
      data = buffer;
      received = true;
    }
  }
  while(size > 0);
  return received;
}
