/** 
* @file Platform/linux/GameHandler.h
* The file declares a class to communicate with the GameController.
* @author <A href="mailto:Thomas.Roefer@dfki.de">Thomas Röfer</A>
*/

#ifndef	__GameHandler_h_
#define	__GameHandler_h_

#define GAME_CONTROL \
  GameHandler theGameHandler;
#define START_GAME_CONTROL \
  theGameHandler.start(GAMECONTROLLER_PORT);
#define RECEIVE_GAME_CONTROL(data) \
  theGameHandler.receive(data)

#include "Representations/Infrastructure/RoboCupGameControlData.h"

/** 
* @class GameHandler
* A class for team communication by broadcasting.
*/
class GameHandler
{
public:
  /**
  * Constructor.
  */
  GameHandler() : port(0) {}

  /**
  * Destructor.
  */
  ~GameHandler();

  /**
  * The method starts the actual communication on the given port.
  * @param port The UDP port this handler is listening to.
  */
  void start(int port);

  /**
  * The method sends the outgoing message queue if possible.
  */
  void send();

  /**
  * The method receives packages if available.
  * @param data If a new package was received, it will be written here.
  */
  bool receive(RoboCup::RoboCupGameControlData& data);

private:
  int port; /**< The UDP port this handler is listening to. */
  int udpSocket; /**< The socket used to communicate. */
};

#endif 
