/** 
* @file Blackboard.cpp
* Implementation of a class representing the blackboard.
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#include "Blackboard.h"
#include <string.h>
#include <stdlib.h>

Blackboard::Blackboard() :
  // Initialize all representations by themselves:
  // Infrastructure
  theJointData(theJointData),
  theSafeJointData(theSafeJointData),
  theJointRequest(theJointRequest),
  theSensorData(theSensorData),
  theSafeSensorData(theSafeSensorData),
  theSensorSAPOutput(theSensorSAPOutput),
  theKeyStates(theKeyStates),
  theLEDRequest(theLEDRequest),
  theImage(theImage),
  theCameraInfo(theCameraInfo),
  theFrameInfo(theFrameInfo),
  theRobotInfo(theRobotInfo),
  theOwnTeamInfo(theOwnTeamInfo),
  theOpponentTeamInfo(theOpponentTeamInfo),
  theGameInfo(theGameInfo),
  theTeamStatus(theTeamStatus),
  theSoundRequest(theSoundRequest),
  theSoundOutput(theSoundOutput),
  theTeamMateData(theTeamMateData),
  theMotionRobotHealth(theMotionRobotHealth),
  theRobotHealth(theRobotHealth),

  // Configuration
  theRobotName(theRobotName),
  theColorTable64(theColorTable64),
  theCameraSettings(theCameraSettings),
  theFieldDimensions(theFieldDimensions),
  theRobotDimensions(theRobotDimensions),
  theJointCalibration(theJointCalibration),
  theSensorCalibration(theSensorCalibration),
  theCameraCalibration(theCameraCalibration),
  theBehaviorConfiguration(theBehaviorConfiguration),
  theWalkingParameters(theWalkingParameters),
  theBalanceParameters(theBalanceParameters),

  // Perception
  theCameraMatrix(theCameraMatrix),
  theImageCoordinateSystem(theImageCoordinateSystem),
  theBallSpot(theBallSpot),
  theBallPercept(theBallPercept),
  thePointsPercept(thePointsPercept),
  theSegmentsPercept(theSegmentsPercept),

  // Modeling
  theFallDownState(theFallDownState),
  theBallModel(theBallModel),
  theGroundTruthBallModel(theGroundTruthBallModel),
  theGoalModel(theGoalModel),
  theFreeSpaceModel(theFreeSpaceModel),
  theRobotPose(theRobotPose),
  theGroundTruthRobotPose(theGroundTruthRobotPose),
  theDribbleModel(theDribbleModel),
  theSelfLocatorParameter(theSelfLocatorParameter),

  // BehaviorControl
  theBehaviorControlOutput(theBehaviorControlOutput),

  // MotionControl
  theOdometryData(theOdometryData),
  theMotionRequest(theMotionRequest),
  theHeadMotionRequest(theHeadMotionRequest),
  theMotionSelection(theMotionSelection),
  theSpecialActionsOutput(theSpecialActionsOutput),
  theWalkingEngineOutput(theWalkingEngineOutput),
  theWalkingEngineStandOutput(theWalkingEngineStandOutput),
  theMotionInfo(theMotionInfo)
{
}

void Blackboard::operator=(const Blackboard& other)
{
  memcpy(this, &other, sizeof(Blackboard)); 
}

void* Blackboard::operator new(unsigned size)
{
  return calloc(1, size);
}

void Blackboard::operator delete(void* p)
{
  return free(p);
}

void Blackboard::distract()
{
}

PROCESS_WIDE_STORAGE Blackboard* Blackboard::theInstance = 0;
