/**
* @file BalanceParameters.h
* @author <a href="mailto:judy@informatik.uni-bremen.de">Judith Mller</a>
*/

#ifndef __BalanceParameters_H__
#define __BalanceParameters_H__

#include "Tools/Streams/StreamHandler.h"



class BalanceParameters : public Streamable
{
public:
  double hipPFactor,
    hipIFactor,
    hipDFactor,
    hipUpperLimit,
    footForwardPFactor,
    footForwardIFactor,
    footForwardDFactor,
    footForwardUpperLimit,
    footSidewardPFactor,
    footSidewardIFactor,
    footSidewardDFactor,
    footSidewardUpperLimit,
    kneePFactor,
    kneeIFactor,
    kneeDFactor,
    kneeUpperLimit,
    armPFactor,
    armIFactor,
    armDFactor,
    armUpperLimit;



  /**
  * The method serializes this object.
  * @param in Points to an In-stream, when currently reading.
  * @param out Points to an Out-stream, when currently writing.
  */
  void serialize(In* in, Out* out)
  {

    STREAM_REGISTER_BEGIN(); 
    STREAM(hipPFactor);
    STREAM(hipIFactor);
    STREAM(hipDFactor);
    STREAM(hipUpperLimit);
    STREAM(footForwardPFactor);
    STREAM(footForwardIFactor);
    STREAM(footForwardDFactor);
    STREAM(footForwardUpperLimit);
    STREAM(footSidewardPFactor);
    STREAM(footSidewardIFactor);
    STREAM(footSidewardDFactor);
    STREAM(footSidewardUpperLimit);
    STREAM(kneePFactor);
    STREAM(kneeIFactor);
    STREAM(kneeDFactor);
    STREAM(kneeUpperLimit);
    STREAM(armPFactor);
    STREAM(armIFactor);
    STREAM(armDFactor);
    STREAM(armUpperLimit);


    STREAM_REGISTER_FINISH();    
  }

  /**
  * Constructor.
  * For parameter describtions, see above.
  */
  BalanceParameters():
    hipPFactor(0),
    hipIFactor(0),
    hipDFactor(0),
    hipUpperLimit(0),
    footForwardPFactor(0),
    footForwardIFactor(0),
    footForwardDFactor(0),
    footForwardUpperLimit(0),
    footSidewardPFactor(0),
    footSidewardIFactor(0),
    footSidewardDFactor(0),
    footSidewardUpperLimit(0),
    kneePFactor(0),
    kneeIFactor(0),
    kneeDFactor(0),
    kneeUpperLimit(0),
    armPFactor(0),
    armIFactor(0),
    armDFactor(0),
    armUpperLimit(0)
  {};
};


#endif // __BalanceParameters_H__
