/**
* @file CameraCalibration.h
* Declaration of a class for representing the calibration values of the PDA camera.
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#ifndef __CameraCalibration_H__
#define __CameraCalibration_H__

#include "Tools/Streams/Streamable.h"

class CameraCalibration : public Streamable
{
private:
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM(bodyTiltCorrection);
    STREAM(cameraTiltCorrection);
    STREAM(cameraRollCorrection);
    STREAM_REGISTER_FINISH()
  }

public:
  double bodyTiltCorrection, /**< The correction of the body tilt angle in radians. */
         cameraTiltCorrection, /**< The correction of the camera tilt angle in radians. */
         cameraRollCorrection; /**< The correction of the camera roll angle in radians. */

  /** 
  * Default constructor.
  */
  CameraCalibration() : bodyTiltCorrection(0.0), cameraTiltCorrection(0.0), cameraRollCorrection(0.0) {}        
};

#endif
