/**
 * @file RobotDimensions.h
 *
 * Description of the Dimensions of the Kondo Robot
 *
 * @author Cord Niehaus
 */

#ifndef __RobotDimensions_H__
#define __RobotDimensions_H__

#include "Tools/Streams/Streamable.h"

class RobotDimensions : public Streamable
{
private:
  virtual void serialize(In* in, Out* out)
  { 
    STREAM_REGISTER_BEGIN();

    STREAM(lengthBetweenLegs);
    STREAM(lengthHipToLegJoint2);
    STREAM(upperLegLength);
    STREAM(lowerLegLength);
    STREAM(lengthFootToLegJoint4);
    STREAM(heightLeg5Joint);
 
    STREAM(xLegJoint1ToHeadPan);
    STREAM(zLegJoint1ToHeadPan);
    STREAM(xHeadPanToHeadTilt);
    STREAM(zHeadPanToHeadTilt);
    STREAM(xHeadTiltToCamera);
    STREAM(zHeadTiltToCamera);

    STREAM(motionCycleTime);         
    STREAM(imageRecordingTime);

    STREAM_REGISTER_FINISH();
  }

public:

  double lengthBetweenLegs;         //!<length between leg joints LL1 and LR1
  double lengthHipToLegJoint2;      //!<length between leg joints LL1 and LL2 in z-direction
  double upperLegLength;            //!<length between leg joints LL2 and LL3 in z-direction
  double lowerLegLength;            //!<length between leg joints LL3 and LL4 in z-direction
  double lengthFootToLegJoint4;     //!<length between leg joints LL4 and LL5 in z-direction

  double heightLeg5Joint;           //!<height of leg joints LL4 and LR4 of the ground
 
  double xLegJoint1ToHeadPan;       //!<forward offset between LL1 and head pan joint 
  double zLegJoint1ToHeadPan;       //!<height offset between LL1 and head pan joint 
  double xHeadPanToHeadTilt;        //!<forward offset between head pan joint and head tilt joint
  double zHeadPanToHeadTilt;        //!<height offset between head pan joint and head tilt joint
  double xHeadTiltToCamera;         //!<forward offset between head tilt joint and camera
  double zHeadTiltToCamera;         //!<height offset between head tilt joint and camera

  double motionCycleTime;           //! Length of one motion cycle in seconds.
  double imageRecordingTime;        //! Time the camera requires to take an image (in s, for motion compensation). 
};

#endif
