/**
* @file WalkingParameters.cpp
* Declaration of a class for representing the walking parameters of a Robot.
* @author <a href="mailto:sancho@informatik.uni-bremen.de">Dario Sancho-Pradel</a>
*/

#include <cstring>
#include "WalkingParameters.h"

WalkingParameters& WalkingParameters::operator=(const WalkingParameters& other)
{
  // copy static part, but exclude pointers
  /*memcpy((char*) &stepDuration, (const char*) &other.stepDuration, 
         (char*) &footPositions[0] - (char*) &stepDuration);*/
  this->stepDuration = other.stepDuration;
  this->maxSpeedChange = Pose2D(other.maxSpeedChange);
  this->odometryScale = Vector2<double>(other.odometryScale);
  this->odometryRotationScaleCCW = other.odometryRotationScaleCCW;
  this->odometryRotationScaleCW = other.odometryRotationScaleCW;
  this->maxForwardSpeed = other.maxForwardSpeed;
  this->maxBackwardSpeed = other.maxBackwardSpeed;
  this->maxSidewaysSpeed = other.maxSidewaysSpeed;
  this->maxRotationSpeed = other.maxRotationSpeed;
  this->elbowAngle = other.elbowAngle;
  this->armDeactivationRatio = other.armDeactivationRatio;
  this->bodyTiltOrigin = other.bodyTiltOrigin;
  this->bodyTiltStandOrigin = other.bodyTiltStandOrigin;
  this->bodyTiltSpeedFactor = other.bodyTiltSpeedFactor;
  this->bodyTiltCorrection = other.bodyTiltCorrection;
  this->bodyShiftCorrection = other.bodyShiftCorrection;
  this->bodyShiftToAccel = other.bodyShiftToAccel;
  this->footPanCorrection = other.footPanCorrection;
  this->footRollCorrection = other.footRollCorrection;
  this->bodyShiftSidewaysOriginFactorLeft = other.bodyShiftSidewaysOriginFactorLeft;
  this->bodyShiftSidewaysOriginFactorRight = other.bodyShiftSidewaysOriginFactorRight;
  this->stepHeight = other.stepHeight;
  this->bodyShiftScale = other.bodyShiftScale;
  this->stepShape = other.stepShape;
  this->stepOrigin = Vector3<double>(other.stepOrigin);
  this->stepTiltForward = other.stepTiltForward;
  this->stepTiltBackward = other.stepTiltBackward;
  this->stepTiltSideways = other.stepTiltSideways;
  memcpy(this->stepPhases, other.stepPhases, 3*sizeof(double));
  this->bodyShiftShape = other.bodyShiftShape;
  this->bodyShiftOrigin = other.bodyShiftOrigin;
  this->bodyShiftScaleMode = other.bodyShiftScaleMode;
  this->bodyShiftPhaseShift = other.bodyShiftPhaseShift;
  memcpy(this->bodyShiftPhases, other.bodyShiftPhases, 3*sizeof(double));
  this->rotationShape = other.rotationShape;
  this->rotationOrigin = other.rotationOrigin;
  this->rotationScaleMode = other.rotationScaleMode;
  this->rotationScale = other.rotationScale;
  this->rotationPhaseShift = other.rotationPhaseShift;
  memcpy(this->rotationPhases, other.rotationPhases, 3*sizeof(double));
  this->armShape = other.armShape;
  this->armOrigin = other.armOrigin;
  this->armScaleMode = other.armScaleMode;
  this->armScale = other.armScale;
  this->armPhaseShift = other.armPhaseShift;
  memcpy(this->armPhases, other.armPhases, 3*sizeof(double));
  
  this->init();
  return *this;
}

bool WalkingParameters::operator==(const WalkingParameters& other) const
{
  // copy static part, but exclude pointers
  /*memcpy((char*) &stepDuration, (const char*) &other.stepDuration, 
         (char*) &footPositions[0] - (char*) &stepDuration);*/
  if(
    this->stepDuration == other.stepDuration &&
    this->maxSpeedChange == other.maxSpeedChange &&
    this->odometryScale == other.odometryScale &&
    this->odometryRotationScaleCCW == other.odometryRotationScaleCCW &&
    this->odometryRotationScaleCW == other.odometryRotationScaleCW &&
    this->maxForwardSpeed == other.maxForwardSpeed &&
    this->maxBackwardSpeed == other.maxBackwardSpeed &&
    this->maxSidewaysSpeed == other.maxSidewaysSpeed &&
    this->maxRotationSpeed == other.maxRotationSpeed &&
    this->elbowAngle == other.elbowAngle &&
    this->armDeactivationRatio == other.armDeactivationRatio &&
    this->bodyTiltOrigin == other.bodyTiltOrigin &&
    this->bodyTiltStandOrigin == other.bodyTiltStandOrigin &&
    this->bodyTiltSpeedFactor == other.bodyTiltSpeedFactor &&
    this->bodyTiltCorrection == other.bodyTiltCorrection &&
    this->bodyShiftCorrection == other.bodyShiftCorrection &&
    this->bodyShiftToAccel == other.bodyShiftToAccel &&
    this->footPanCorrection == other.footPanCorrection &&
    this->footRollCorrection == other.footRollCorrection &&
    this->bodyShiftSidewaysOriginFactorLeft == other.bodyShiftSidewaysOriginFactorLeft &&
    this->bodyShiftSidewaysOriginFactorRight == other.bodyShiftSidewaysOriginFactorRight &&
    this->stepHeight == other.stepHeight &&
    this->bodyShiftScale == other.bodyShiftScale &&
    this->stepShape == other.stepShape &&
    this->stepOrigin == other.stepOrigin &&
    this->stepTiltForward == other.stepTiltForward &&
    this->stepTiltBackward == other.stepTiltBackward &&
    this->stepTiltSideways == other.stepTiltSideways &&
    !memcmp(this->stepPhases, other.stepPhases, 3*sizeof(double)) &&
    this->bodyShiftShape == other.bodyShiftShape &&
    this->bodyShiftOrigin == other.bodyShiftOrigin &&
    this->bodyShiftScaleMode == other.bodyShiftScaleMode &&
    this->bodyShiftPhaseShift == other.bodyShiftPhaseShift &&
    !memcmp(this->bodyShiftPhases, other.bodyShiftPhases, 3*sizeof(double)) &&
    this->rotationShape == other.rotationShape &&
    this->rotationOrigin == other.rotationOrigin &&
    this->rotationScaleMode == other.rotationScaleMode &&
    this->rotationScale == other.rotationScale &&
    this->rotationPhaseShift == other.rotationPhaseShift &&
    !memcmp(this->rotationPhases, other.rotationPhases, 3*sizeof(double)) &&
    this->armShape == other.armShape &&
    this->armOrigin == other.armOrigin &&
    this->armScaleMode == other.armScaleMode &&
    this->armScale == other.armScale &&
    this->armPhaseShift == other.armPhaseShift &&
    !memcmp(this->armPhases, other.armPhases, 3*sizeof(double))
    )
     return true;
  else
    return false;
}

void WalkingParameters::serialize(In* in, Out* out)
{
  STREAM_REGISTER_BEGIN();

  STREAM(stepDuration);
  STREAM(maxSpeedChange);
  STREAM(odometryScale);
  STREAM(odometryRotationScaleCCW);
  STREAM(odometryRotationScaleCW);
  STREAM(maxForwardSpeed);
  STREAM(maxBackwardSpeed);
  STREAM(maxSidewaysSpeed);
  STREAM(maxRotationSpeed);
  STREAM(elbowAngle);
  STREAM(armDeactivationRatio);
  STREAM(bodyTiltOrigin);
  STREAM(bodyTiltStandOrigin);
  STREAM(bodyTiltSpeedFactor);
  STREAM(bodyTiltCorrection);
  STREAM(bodyShiftCorrection);
  STREAM(bodyShiftToAccel);
  STREAM(footPanCorrection);
  STREAM(footRollCorrection);

  STREAM_ENUM(stepShape, Trajectory3D::numberOfShapes, &Trajectory3D::getShapeName);
  STREAM(stepOrigin);
  STREAM(stepTiltForward);
  STREAM(stepTiltBackward);
  STREAM(stepTiltSideways);
  STREAM(stepHeight);
  STREAM_ARRAY(stepPhases);

  STREAM_ENUM(bodyShiftShape, Trajectory1D::numberOfShapes, &Trajectory1D::getShapeName);
  STREAM(bodyShiftOrigin);
  STREAM_ENUM(bodyShiftScaleMode, Trajectory1D::numberOfScaleModes, &Trajectory1D::getScaleModeName);
  STREAM(bodyShiftScale);
  STREAM(bodyShiftPhaseShift);
  STREAM_ARRAY(bodyShiftPhases);
  STREAM(bodyShiftSidewaysOriginFactorLeft);
  STREAM(bodyShiftSidewaysOriginFactorRight);

  STREAM_ENUM(rotationShape, Trajectory1D::numberOfShapes, &Trajectory1D::getShapeName);
  STREAM(rotationOrigin);
  STREAM_ENUM(rotationScaleMode, Trajectory1D::numberOfScaleModes, &Trajectory1D::getScaleModeName);
  STREAM(rotationScale);
  STREAM(rotationPhaseShift);
  STREAM_ARRAY(rotationPhases);

  STREAM_ENUM(armShape, Trajectory1D::numberOfShapes, &Trajectory1D::getShapeName);
  STREAM(armOrigin);
  STREAM_ENUM(armScaleMode, Trajectory1D::numberOfScaleModes, &Trajectory1D::getScaleModeName);
  STREAM(armScale);
  STREAM(armPhaseShift);
  STREAM_ARRAY(armPhases);

  STREAM_REGISTER_FINISH();

  if(in)
    this->init();
}

void WalkingParameters::init()
{
  this->free();

  footPositions[0] = Trajectory3D::create(stepShape, stepOrigin, 
                                          stepTiltForward, stepTiltBackward, stepTiltSideways, 
                                          0, stepHeight, stepPhases);
  footPositions[1] = Trajectory3D::create(stepShape, 
                                          Vector3<double>(stepOrigin.x, -stepOrigin.y, stepOrigin.z), 
                                          stepTiltForward, stepTiltBackward, stepTiltSideways, 
                                          0.5, stepHeight, stepPhases);
  bodyShift = Trajectory1D::create(bodyShiftShape, bodyShiftOrigin, bodyShiftScaleMode, bodyShiftScale, bodyShiftPhaseShift, bodyShiftPhases);
  rotation[0] = Trajectory1D::create(rotationShape, rotationOrigin, rotationScaleMode, rotationScale, rotationPhaseShift, rotationPhases);
  rotation[1] = Trajectory1D::create(rotationShape, rotationOrigin, rotationScaleMode, rotationScale, rotationPhaseShift + 0.5, rotationPhases);
  arm[0] = Trajectory1D::create(armShape, armOrigin, armScaleMode, armScale, armPhaseShift, armPhases);
  arm[1] = Trajectory1D::create(armShape, armOrigin, armScaleMode, armScale, armPhaseShift + 0.5, armPhases);
}

void WalkingParameters::free()
{
  if(footPositions[0])
  {
    for(int side = 0; side < 2; ++side)
    {
      delete footPositions[side];
      delete rotation[side];
      delete arm[side];
    }
    delete bodyShift;
    footPositions[0] = 0;
  }
}

void WalkingParameters::setOptParameters(const struct GaitOptParameters& params)
{
  this->stepDuration = params.stepDuration;
  this->bodyTiltOrigin = params.bodyTilt;
  this->stepPhases[0] = params.stepPhase[0];
  this->stepPhases[1] = params.stepPhase[1];
  this->stepPhases[2] = params.stepPhase[2];
  this->stepHeight = params.stepHeight;
  this->stepTiltForward = params.stepTiltForward;
  this->stepTiltBackward = params.stepTiltBackward;
  this->stepTiltSideways = params.stepTiltSideways;
  this->stepOrigin[0] = params.stepOrigin[0];
  this->stepOrigin[1] = params.stepOrigin[1];
  this->stepOrigin[2] = params.stepOrigin[2];
  this->bodyShiftPhases[0] = params.bodyShiftPhases[0];
  this->bodyShiftPhases[1] = params.bodyShiftPhases[1];
  this->bodyShiftPhases[2] = params.bodyShiftPhases[2];
  this->bodyShiftPhaseShift = params.bodyShiftPhaseShift;
  this->armOrigin = params.armOrigin;
  this->armScale = params.armScale;

  this->init();
}
