/**
 * @file SoundRequest.h
 * Declaration of class SoundRequest.
 * @author Philippe Schober
 */ 

#ifndef __SoundRequest_h_
#define __SoundRequest_h_

#include "Tools/Streams/Streamable.h"

class SoundRequest : public Streamable
{
private:
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
      STREAM_ENUM(sound, numOfSounds, SoundRequest::getSoundName);
      STREAM(once);
    STREAM_REGISTER_FINISH();
  }

public:
  enum Sound
  {
    none,
    connected,
    startup,
    shutdown,
    marioHereWeGo,
    marioLetsGo,
    marioHighScore,
    marioGameOver,
    marioInjury,
    marioGotIt,
    marioMamamia,
    _initial,
    ready,
    set,
    playing,
    yellow,
    blue,
    striker,
    penaltyStriker,
    defender,
    challenger,
    ownKickoff,
    opponentKickoff,
    keeper,
    penaltyKeeper,
    supporter,
    passing,
    network,
    one,
    zero,
    numOfSounds
  };

  Sound sound; /**< The requested sound to be played. */
  bool once; /**< Play sound only once. */
 
  static const char* getSoundName(Sound sound)
  {
    switch(sound)
    {
      case none: return "none";
      case connected: return "connected";
      case startup: return "startup";
      case shutdown: return "shutdown";
      case marioHereWeGo: return "marioHereWeGo";
      case marioLetsGo: return "marioLetsGo";
      case marioHighScore: return "marioHighScore";
      case marioGameOver: return "marioGameOver";
      case marioInjury: return "marioInjury";
      case marioGotIt: return "marioGotIt";
      case marioMamamia: return "marioMamamia";
      case _initial: return "_initial";
      case ready: return "ready";
      case set: return "set";
      case playing: return "playing";
      case yellow: return "yellow";
      case blue: return "blue";
      case striker: return "striker";
      case penaltyStriker: return "penalty_striker";
      case defender: return "defender";
      case challenger: return "challenger";
      case ownKickoff: return "ownKickoff";
      case opponentKickoff: return "opponentKickoff";
      case keeper: return "keeper";
      case penaltyKeeper: return "penalty_keeper";
      case supporter: return "supporter";
      case passing: return "passing";
      case network: return "network";
      case one: return "one";
      case zero: return "zero";
      default: return "unknown";
    }
  };

  /**
  * The function returns the filename of the sound to play.
  * @param sound The sound.
  * @return The filename of this sound.
  */
  static const char* getSoundFilename(Sound sound)
  {
    switch(sound)
    {
      case connected: return "connected.wav";
      case startup: return "femstartup.wav";
      case shutdown: return "femshutdown.wav";
      case marioHereWeGo: return "sm64_here_we_go.wav";
      case marioLetsGo: return "sm64_lets_go.wav";
      case marioHighScore: return "sm64_high_score.wav";
      case marioGameOver: return "sm64_game_over.wav";
      case marioInjury: return "sm64_injury.wav";
      case marioGotIt: return "mario_i_got_it.wav";
      case marioMamamia: return "mario_mamamia.wav";
      case _initial: return "initial.wav";
      case ready: return "ready.wav";
      case set: return "set.wav";
      case playing: return "playing.wav";
      case yellow: return "yellow.wav";
      case blue: return "blue.wav";
      case striker: return "striker.wav";
      case penaltyStriker: return "penalty_striker.wav";
      case defender: return "defender.wav";
      case challenger: return "challenger.wav";
      case ownKickoff: return "own_kickoff.wav";
      case opponentKickoff: return "opponent_kickoff.wav";
      case keeper: return "keeper.wav";
      case penaltyKeeper: return "penalty_keeper.wav";
      case supporter: return "supporter.wav";
      case passing: return "passing.wav";
      case network: return "network.wav";
      case one: return "one.wav";
      case zero: return "zero.wav";
      default: return 0;
    }
  };

  /**
  * Default constructor.
  */
  SoundRequest() : sound(none), once(true) {}
};

/**
* A dummy class so SoundControl is selectable.
*/
class SoundOutput : public Streamable
{
private:
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
      char dummy(0);
      STREAM(dummy);
    STREAM_REGISTER_FINISH();
  }
};

#endif //__SoundRequest_h_
