/**
* \file TeamMateData.h
* Declaration of a class representing information about the teammates.
* \author Colin Graf
*/ 

#ifndef __TeamMateData_h_
#define __TeamMateData_h_

#include "Representations/Modeling/RobotPose.h"
#include "Representations/Modeling/BallModel.h"
#include "Representations/Infrastructure/BehaviorData.h"

/**
* \class TeamMateData
* A class representing information about the teammates.
*/ 
class TeamMateData : public Streamable  
{
private:
  virtual void serialize(In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN();
    STREAM(numOfConnectedPlayers);
    STREAM(sendThisFrame);
    STREAM(wasConnected);
    STREAM_ARRAY(timeStamps);
    STREAM_ARRAY(ballModels);
    STREAM_ARRAY(robotPoses);
    STREAM_ARRAY(behaviorData);
    STREAM_REGISTER_FINISH();
  }

public:
  enum Player
  {
    firstPlayer=1,
    player1=1,
    player2,
    player3,
    player4,
    numOfPlayers
  };
  unsigned numOfConnectedPlayers; /**< The number of robots of which messages were received recently. _Not_ including this robot itself. */
  bool sendThisFrame; /**< The team communication will be sent in this frame. */
  bool wasConnected; /**< Whether we have been connected to a team mate. */
  unsigned int timeStamps[numOfPlayers]; /**< The times when messages from different robots arrived. */
  BallModel ballModels[numOfPlayers]; /**< The last received ball model of each team mate. */
  RobotPose robotPoses[numOfPlayers]; /**< The last received robot pose of each team mate. */
  BehaviorData behaviorData[numOfPlayers]; /**< The last received behavior data of each team mate. */

  /**
  * Default constructor.
  */
  TeamMateData() : 
    numOfConnectedPlayers(0), 
    sendThisFrame(false),
    wasConnected(false)
  {
    for(int i = 0; i < numOfPlayers; ++i)
      timeStamps[i] = 0;
  }
};

#endif //__TeamMateData_h_
