/**
* @file DribbleModel.h
*
* Declaration of class DribbleModel
*
* @author <a href="mailto:rieskamp@informatik.uni-bremen.de">Andrik Rieskamp</a>
*/

#ifndef __DribbleModel_h_
#define __DribbleModel_h_

#include "Tools/Streams/Streamable.h"
#include "Tools/Math/Vector2.h"

/**
* @class DribbleModel
*
* A class that represents the current state of the robot's environment
*/
class DribbleModel : public Streamable
{
private:
  /** Streaming function 
  * @param in Object for streaming in the one direction
  * @param out Object for streaming in the other direction
  */
  void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM(goLeftAroundPole);
    STREAM(poleIsInFront);
    STREAM(angleToPole1);
    STREAM(angleToPole2);
    STREAM(angleToPole3);
    STREAM(distanceToPole1);
    STREAM(distanceToPole2);
    STREAM(distanceToPole3);
    STREAM(angleToPass1);
    STREAM(angleToPass2);    
    STREAM(distanceToPass1);
    STREAM(distanceToPass2);
    STREAM(timeSincePoleLastSeen1);
    STREAM(timeSincePoleLastSeen2);
    STREAM(timeSincePoleLastSeen3);
    STREAM_REGISTER_FINISH();
  }

public:
  bool goLeftAroundPole, poleIsInFront;
  double angleToPole1, angleToPole2, angleToPole3;
  double distanceToPole1, distanceToPole2, distanceToPole3;
  double angleToPass1, angleToPass2;
  double distanceToPass1, distanceToPass2;
  double timeSincePoleLastSeen1, timeSincePoleLastSeen2, timeSincePoleLastSeen3;

  /** Default constructor. */
  DribbleModel():
    goLeftAroundPole(false),
    poleIsInFront(false),
    angleToPole1(0.0),
    angleToPole2(0.0),
    angleToPole3(0.0),
    distanceToPole1(0.0),
    distanceToPole2(0.0),
    distanceToPole3(0.0),
    angleToPass1(0.0),
    angleToPass2(0.0),
    distanceToPass1(0.0),
    distanceToPass2(0.0),
    timeSincePoleLastSeen1(0.0),
    timeSincePoleLastSeen2(0.0),
    timeSincePoleLastSeen3(0.0)
    {}
};

#endif //__DribbleModel_h_
