/**
* @file FreeSpaceModel.h
*
* Declaration of class FreeSpaceModel
*
* @author <A href="mailto:timlaue@informatik.uni-bremen.de">Tim Laue</A>
* @author <a href="mailto:rieskamp@informatik.uni-bremen.de">Andrik Rieskamp</a>
*/

#ifndef __FreeSpaceModel_h_
#define __FreeSpaceModel_h_

#include "Tools/Streams/Streamable.h"


/**
* @class FreeSpaceModel
*
* A class that represents the current state of the robot's environment
*/
class FreeSpaceModel : public Streamable
{
private:
  /** Streaming function 
  * @param in Object for streaming in the one direction
  * @param out Object for streaming in the other direction
  */
  void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
      STREAM(frontIsFree2);
      STREAM(freeDistanceInFrontOfMe2);
      STREAM(bestAngle);
      STREAM(freeGoalPartInFrontOfMe);
      STREAM(timeSinceFrontIsFree);
      STREAM(bestObstacleAvoidingAngle);
    STREAM_REGISTER_FINISH();
  }

public:
  /** Current state of the environment at the robot's front*/
  bool frontIsFree2;

  /** Distance to obstacles in front of me within a specific width */
  double freeDistanceInFrontOfMe2;

  /** Best free angle calculated from goal angle AND freeSpaceSegments */
  double bestAngle;

  /** The time since the front of the robot is not blocked */
  double timeSinceFrontIsFree;

  /** The best angle to avoid the obstacle in front of me */
  double bestObstacleAvoidingAngle;

  /** Is a sufficent angle in front of the robot free and the goal visible there */
  bool freeGoalPartInFrontOfMe;

  /** Default constructor. */
  FreeSpaceModel():
    frontIsFree2(true), 
    freeDistanceInFrontOfMe2(0.0), 
    bestAngle(0.0),
    timeSinceFrontIsFree(0.0),
    bestObstacleAvoidingAngle(0.0),
    freeGoalPartInFrontOfMe(false)
    {}
};

#endif //__FreeSpaceModel_h_
