/**
* @file RobotPose.cpp
*
* contains the implementation of the streaming operators 
* for the class RobotPose
*/

#include "RobotPose.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Math/Geometry.h"


void RobotPose::draw()
{
  Vector2<double> bodyPoints[4] = {Vector2<double>(55,90),
                                   Vector2<double>(-55,90),
                                   Vector2<double>(-55,-90),
                                   Vector2<double>(55,-90)};
  for(int i=0; i<4; i++)
  {
    bodyPoints[i] = Geometry::rotate(bodyPoints[i], rotation);
    bodyPoints[i] += translation;
  }
  Vector2<double> headPos(30,0);
  headPos = Geometry::rotate(headPos, rotation);
  headPos += translation;
  DECLARE_DEBUG_DRAWING("representation:RobotPose", "drawingOnField", "The robot pose");
  POLYGON("representation:RobotPose", 4, bodyPoints, 
      1, // pen width 
      Drawings::ps_solid, 
      ColorClasses::black,
      Drawings::bs_solid, 
      ColorClasses::black);
  CIRCLE("representation:RobotPose", 
      headPos.x, 
      headPos.y, 
      30, 
      1, // pen width 
      Drawings::ps_solid, 
      ColorRGBA(180,180,180),
      Drawings::bs_solid, 
      ColorRGBA(180,180,180));

  DECLARE_DEBUG_DRAWING("origin:RobotPose", "drawingOnField", "Set the origin to the robot's current position.");
  ORIGIN("origin:RobotPose", translation.x, translation.y, rotation);
}

void GroundTruthRobotPose::draw()
{
  Vector2<double> bodyPoints[4] = {Vector2<double>(55,90),
                                   Vector2<double>(-55,90),
                                   Vector2<double>(-55,-90),
                                   Vector2<double>(55,-90)};
  for(int i=0; i<4; i++)
  {
    bodyPoints[i] = Geometry::rotate(bodyPoints[i], rotation);
    bodyPoints[i] += translation;
  }
  Vector2<double> headPos(30,0);
  headPos = Geometry::rotate(headPos, rotation);
  headPos += translation;
  DECLARE_DEBUG_DRAWING("representation:GroundTruthRobotPose", "drawingOnField", "The ground truth robot pose");
  POLYGON("representation:GroundTruthRobotPose", 4, bodyPoints, 
      1, // pen width 
      Drawings::ps_solid, 
      ColorRGBA(0, 0, 0, 128),
      Drawings::bs_solid, 
      ColorRGBA(0, 0, 0, 128));
  CIRCLE("representation:GroundTruthRobotPose", 
      headPos.x, 
      headPos.y, 
      30, 
      1, // pen width 
      Drawings::ps_solid, 
      ColorRGBA(180,180,180),
      Drawings::bs_solid, 
      ColorRGBA(180,180,180));

  DECLARE_DEBUG_DRAWING("origin:GroundTruthRobotPose", "drawingOnField", "Set the origin to the robot's ground truth position.");
  ORIGIN("origin:GroundTruthRobotPose", translation.x, translation.y, rotation);
}
