/**
 * @file RobotPose.h
 *
 * The file contains the definition of the class RobotPose.
 *
 * @author <A href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</A>
 */

#ifndef __RobotPose_h_
#define __RobotPose_h_

#include "Tools/Math/Pose2D.h"

/**
* @class RobotPose
* A Pose2D with validity.
*/
class RobotPose : public Pose2D
{
private:
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM_BASE(Pose2D);
    STREAM(validity);
    STREAM_REGISTER_FINISH();
  }

public:
  double validity; /**< The validity of the robot pose. */
 
  /** Constructor */
  RobotPose():validity(0.0) {}

  /** Assignment operator
  * @param other Another RobotPose
  * @return A reference to the object after the assignment
  */
  const RobotPose& operator=(const RobotPose& other)
  {
    (Pose2D&) *this = (const Pose2D&) other;
    validity = other.validity;
    return *this;
  }

  /** Cast Contructor.
  * @param otherPose A Pose2D object
  */
  RobotPose (const Pose2D& otherPose) 
  {
    (Pose2D&) *this = otherPose;
    validity = 0;
  }

  /** Assignment operator for Pose2D objects
  * @param other A Pose2D object
  * @return A reference to the object after the assignment
  */
  const RobotPose& operator=(const Pose2D& other)
  {
    (Pose2D&) *this = other;
    //validity is not set
    return *this;
  }

  /** Draws the robot pose to the field view*/
  void draw();
};

class GroundTruthRobotPose : public RobotPose 
{
public:
  /** Draws the robot pose to the field view*/
  void draw();
};

#if 0
class TeamMateRobotPose : public RobotPose {};

class TeamRobotPoses : public Streamable
{
public:

  RobotPose robotPoses[12]; /**< The last received robot pose of each team mate. (0 = currently unused) */

  /** Streaming (with specifications) */
  virtual void serialize(In *in, Out *out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM(robotPoses[0]);
    STREAM(robotPoses[1]);
    STREAM(robotPoses[2]);
    STREAM(robotPoses[3]);
    STREAM(robotPoses[4]);
    STREAM(robotPoses[5]);
    STREAM(robotPoses[6]);
    STREAM(robotPoses[7]);
    STREAM(robotPoses[8]);
    STREAM(robotPoses[9]);
    STREAM(robotPoses[10]);
    STREAM(robotPoses[11]);
    STREAM_REGISTER_FINISH();
  }
};
#endif //0

#endif //__RobotPose_h_
