#ifndef SELFLOCATORPARAMETER_H
#define SELFLOCATORPARAMETER_H

#include <Tools/Math/Pose2D.h>
#include <Tools/Settings.h>

/**
* A collection of all parameters of the module.
*/
class SelfLocatorParameter : public Streamable
{
private:
  /**
  * The method makes the object streamable.
  * @param in The stream from which the object is read 
  * @param out The stream to which the object is written 
  */
  virtual void serialize(In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN();
    STREAM(standardDeviation);
    STREAM(alphaSlow);
    STREAM(alphaFast);
    STREAM(resamplingThreshold);
    STREAM(beaconTemplateRatio);
    STREAM(translationNoise);
    STREAM(rotationNoise);
    STREAM(movedDistWeight);
    STREAM(movedAngleWeight);
    STREAM(majorDirTransWeight);
    STREAM(minorDirTransWeight);
    STREAM(maxCrossingLength);
    STREAM(maxBeaconLength);
    STREAM(numberOfPoints);
    STREAM(disableSensorResetting);

    STREAM(considerGameState);
    STREAM(knownStartPose);
    STREAM(startPose);
    STREAM(startPoseStandardDeviation);
    STREAM_REGISTER_FINISH();
  }

public:

  /*
  void operator=(const SelfLocatorParameter& p)
  {
    standardDeviation = p.standardDeviation;
    alphaSlow = p.alphaSlow;
    alphaFast = p.alphaFast;
    resamplingThreshold = p.resamplingThreshold;
    beaconTemplateRatio = p.beaconTemplateRatio;
    translationNoise = p.translationNoise;
    rotationNoise = p.rotationNoise;
    movedDistWeight = p.movedDistWeight;
    movedAngleWeight = p.movedAngleWeight;
    majorDirTransWeight = p.majorDirTransWeight;
    minorDirTransWeight = p.minorDirTransWeight;
    maxCrossingLength = p.maxCrossingLength;
    maxBeaconLength = p.maxBeaconLength;
    numberOfPoints = p.numberOfPoints;
    disableSensorResetting = p.disableSensorResetting;
    considerGameState = p.considerGameState;
    knownStartPose = p.knownStartPose;
    startPose = p.startPose;
    startPoseStandardDeviation = p.startPoseStandardDeviation;
  }
  */

  int standardDeviation;
  double alphaSlow;
  double alphaFast;
  double resamplingThreshold;
  int beaconTemplateRatio;
  double translationNoise;
  double rotationNoise;
  double movedDistWeight;
  double movedAngleWeight;
  double majorDirTransWeight;
  double minorDirTransWeight;
  int maxCrossingLength;
  int maxBeaconLength;
  int numberOfPoints;
  bool disableSensorResetting;
  bool considerGameState;
  bool knownStartPose;
  Pose2D startPose;
  Pose2D startPoseStandardDeviation;
};


#endif
