/**
* @file CameraMatrix.h
* 
* Declaration of class CameraMatrix
*/ 

#ifndef __CameraMatrix_h_
#define __CameraMatrix_h_

#include "Tools/Math/Pose3D.h"

/**
* Matrix describing transformation from neck joint to camera.
*/
class CameraMatrix : public Pose3D
{
private:
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM_BASE(Pose3D);
    STREAM(isValid);
    STREAM_REGISTER_FINISH();
  }

public:
  /** Kind of copy-constructor 
  * @param pose The other pose
  */
  CameraMatrix(const Pose3D& pose): Pose3D(pose), isValid(true) {}

  /**Standard constructor*/
  CameraMatrix() : isValid(true) {}

  /** Draws the camera matrix*/
  void draw();

  bool isValid; /**< Matrix is only valid if motion was stable. */
};

#endif //__CameraMatrix_h_
