
#include <QLabel>
#include <QVBoxLayout>
#include <QSlider>
#include <QLineEdit>
#include <QPushButton>
#include <QSpinBox>
#include <QCheckBox>

#include "Document.h"
#include "ActuatorWindow.h"
#include "../SimRobotCore/Simulation/Simulation.h"

ActuatorWidget::ActuatorWidget(const QString& object, QWidget* parent) : QWidget(parent)
{
  actuatorName = object;

  Simulation *sim = Document::document->getSimulation();
  int actuatorPortId = sim->getActuatorportId(actuatorName.toAscii().constData());
  double minVal = sim->getActuatorportMinValue(actuatorPortId);
  double maxVal = sim->getActuatorportMaxValue(actuatorPortId);

  QStringList nameList = object.split(".");

  slider = new QSlider(Qt::Vertical);
  slider->setRange((int)minVal, (int)maxVal);
  slider->setValue(0);

  btnExit = new QPushButton(tr("Close"), this);

  txbValue = new QSpinBox(this);
  txbValue->setValue(0);
  txbValue->setRange((int)minVal, (int)maxVal);

  QLabel* label = new QLabel(nameList[nameList.size() - 2], this);
  label->setFont(QFont("Courier New", 7));

  cbxSet = new QCheckBox(tr("set"), this);

  connect(slider, SIGNAL(valueChanged(int)), this, SLOT(valueChangedSlot(int)));
  connect(txbValue, SIGNAL(valueChanged(int)), this, SLOT(valueChangedSlot(int)));
  connect(btnExit, SIGNAL(released()), this, SLOT(closeSlot()));

  //connect(Document::document, SIGNAL(updateViewsSignal()), this, SLOT(updateSlot()));
  connect(Document::document, SIGNAL(updateActuatorsSignal()), this, SLOT(updateActuatorSlot()));

  QVBoxLayout *layout = new QVBoxLayout();

  layout->addWidget(label);
  layout->addWidget(slider, 0, Qt::AlignHCenter);
  layout->addWidget(txbValue);
  layout->addWidget(cbxSet);
  layout->addWidget(btnExit);

  setLayout(layout);
}

void ActuatorWidget::valueChangedSlot(int value)
{
  txbValue->setValue(value);
  slider->setValue(value);
}

void ActuatorWidget::closeSlot()
{
  Document::document->closeObject(actuatorName);
}

void ActuatorWidget::updateActuatorSlot()
{
  if(cbxSet->checkState() == Qt::Checked)
  {
    Simulation *sim = Document::document->getSimulation();
    int actuatorPortId = sim->getActuatorportId(actuatorName.toAscii().constData());
    sim->setActuatorport(actuatorPortId, (double)slider->value());
  }
}

ActuatorWindow::ActuatorWindow(QMdiArea* mdiArea) : SubWindow(mdiArea, ".Actuator")
{
  setWindowTitle(tr("Actuator Window"));
  setMinimumSize(QSize(200, 150));

  layout = new QHBoxLayout();
  QWidget* widget = new QWidget();
  widget->setLayout(layout);
  setWidget(widget);
}

void ActuatorWindow::addObject(const QString& obj)
{
  if(actuators.contains(obj))
    return;

  ActuatorWidget *widget = new ActuatorWidget(obj, this);
  layout->addWidget(widget);
  actuators[obj] = widget;
}

void ActuatorWindow::removeObject(const QString& obj)
{
  if(!actuators.contains(obj))
    return;

  QWidget* widget = actuators[obj];
  layout->removeWidget(widget);
  actuators.remove(obj);
  widget->setAttribute(Qt::WA_DeleteOnClose);
  widget->close();
}

int ActuatorWindow::getObjectCount()
{
  return actuators.size();
}

void ActuatorWindow::closeEvent(QCloseEvent* event)
{
 for(QHash<QString, ActuatorWidget*>::iterator i = actuators.begin(), next, end = actuators.end(); i != end; i = next)
 {
   next = i;
   ++next;
   QString obj(i.key());
   actuators.remove(obj);
   Document::document->closeObject(obj);
 }

  SubWindow::closeEvent(event);
}

void ActuatorWindow::restoreLayout()
{
  QSettings* settings = Document::document->getLayoutSettings();
  settings->beginGroup(object);
  restoreGeometry(*settings);
  settings->endGroup();
}

void ActuatorWindow::writeLayout()
{
  QSettings* settings = Document::document->getLayoutSettings();
  settings->beginGroup(object);
  writeGeometry(*settings);
  settings->endGroup();
}

