
#include <QHideEvent>
#include <QApplication>
#include <QScrollBar>

#include "Console.h"
#include "MenuBar.h"
#include "Document.h"

Console::Console(QWidget *parent) : QDockWidget(parent)
{
  setObjectName(tr("Console"));
  setWindowTitle(tr("Console"));
  setFeatures(features() | DockWidgetVerticalTitleBar);

  textEdit = new ConsoleTextEdit(this);
  setWidget(textEdit);

  connect(MenuBar::menuBar->viewConsoleAction, SIGNAL(triggered(bool)), SLOT(setVisible(bool)));
  MenuBar::menuBar->viewConsoleAction->setEnabled(true);

  connect(Document::document, SIGNAL(clearConsoleSignal()), this, SLOT(clearSlot()));
  connect(Document::document, SIGNAL(printSignal(const QString&)), this, SLOT(printSlot(const QString&)));
}

void Console::hideEvent(QHideEvent * event)
{
  QDockWidget::hideEvent(event);

  MenuBar::menuBar->viewConsoleAction->setChecked(false);
  event->accept();
}

void Console::showEvent(QShowEvent * event)
{
  QDockWidget::showEvent(event);

  MenuBar::menuBar->viewConsoleAction->setChecked(true);
  event->accept();
}

void Console::clearSlot()
{
  textEdit->setPlainText("");
}

void Console::printSlot(const QString& text)
{
  QScrollBar* scrollBar = textEdit->verticalScrollBar();
  bool scroll = scrollBar->value() == scrollBar->maximum();
  QTextCursor cursor = textEdit->textCursor();
  cursor.movePosition(QTextCursor::StartOfBlock);
  /*
  cursor.movePosition(QTextCursor::NextBlock);
  if(cursor.atEnd() && !cursor.atStart())
  {
    cursor.insertText("\n");
    cursor.movePosition(QTextCursor::NextBlock);
  }*/
  cursor.insertText(text);
  if(scroll)
    scrollBar->setValue(scrollBar->maximum());
  cursor.movePosition(QTextCursor::End);
}

ConsoleTextEdit::ConsoleTextEdit(Console* parent) : QTextEdit(parent)
{
}

void ConsoleTextEdit::keyPressEvent(QKeyEvent* event)
{
  switch(event->key())
  {
  case Qt::Key_Tab:
  case Qt::Key_Backtab:
    event->accept();
    {
      QTextCursor cursor = textCursor();
      int begin = cursor.position();
      int end = cursor.anchor();
      if(end < begin)
      {
        int tmp = end;
        end = begin;
        begin = tmp;
      }
      cursor.setPosition(end);
      cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
      QString line = cursor.selectedText();
      int lineLength = begin - cursor.position();
      Document::document->completeConsoleCommand(line, event->key() == Qt::Key_Tab, begin == end);
      cursor.insertText(line);
      cursor.setPosition(begin);
      cursor.setPosition(begin + (line.length() - lineLength), QTextCursor::KeepAnchor);      
      setTextCursor(cursor);
    }
    break;

  case Qt::Key_Return:
  case Qt::Key_Enter:
    event->accept();
    {
      QTextCursor cursor = textCursor();
      cursor.movePosition(QTextCursor::StartOfBlock);
      cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
      QString line = cursor.selectedText();
      cursor.movePosition(QTextCursor::EndOfLine);
      if(cursor.atEnd())
        cursor.insertText("\n");
      cursor.movePosition(QTextCursor::NextBlock);
      setTextCursor(cursor);
      
      Document::document->handleConsoleCommand(line);      
    }
    break;

  default:
    QTextEdit::keyPressEvent(event);
    break;
  }
}

