/** 
* @file SimRobotGUI/MainWindow.h
* Declaration of class MainWindow.
* @author Colin Graf
*/

#ifndef MainWindow_H
#define MainWindow_H

#include <QMainWindow>
#include <QSignalMapper>

#include "Document.h"
#include "../SimRobotCore/Simulation/APIDatatypes.h"

class QMdiArea;
class QActionGroup;
class QMdiSubWindow;
class QSettings;
class MenuBar;
class ToolBar;
class StatusBar;
class Document;
class Console;
class Tree;
class SubWindow;
class ActuatorWindow;

class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
  MainWindow(int argc, char *argv[]);

private slots:
  void updateWindowMenuSlot();
  void updateFileMenuSlot();
  void aboutSlot();
  void openDocumentSlot();
  void closeDocumentSlot();
  void changeDocumentSlot();
  void saveDocumentSlot();
  void openObjectSlot(const QString& object, DocumentObjectType type);
  void raiseObjectSlot(const QString& object, DocumentObjectType typ);
  void updateObjectSlot(const QString& object, DocumentObjectType type);
  void closeObjectSlot(const QString& object, DocumentObjectType typ);
  void setActiveSubWindowSlot(QWidget *window);
  void subWindowActivatedSlot(QMdiSubWindow*);
  void writeLayoutSlot();
  void restoreLayoutSlot();
  void showEditorSlot(bool show);
  void resizeSubWindow(int size);

private:
  QMdiArea* mdiArea;
  QSignalMapper windowMapper;
  QActionGroup* windowActionGroup;
  MenuBar* menuBar;
  QSignalMapper resizeMapper;
  ToolBar* toolBar;
  StatusBar* statusBar;
  Document* document;
  Console *console;
  Tree *tree;
  ActuatorWindow* actuatorWindow;
  SubWindow* activeSubWindow;
  QRect normalGeometry; /**< The restored or last recoginzed, not maximized or minimized window geometry. */
  QSignalMapper recentFileMapper;

  void createConnections();

  /**
  * Saves the geometry (position, size and state) of the window. (Since QWidget::saveGeometry is buggy.)
  * Use this in writeLayout().
  * @param settings The layout settings used to save the geometry.
  */
  void writeGeometry(QSettings& settings);

  /**
  * Restores the geometry (position, size and state) of the window.
  * Use this in restoreLayout().
  * @param settings The layout settings in which the geomerty was saved.
  */
  void restoreGeometry(QSettings& settings);

  void closeEvent(QCloseEvent* event);
  void moveEvent(QMoveEvent* event);
  void resizeEvent(QResizeEvent* event);
  void dragEnterEvent(QDragEnterEvent *event);
  void dropEvent(QDropEvent *event);
};

#endif // MainWindow_H
