
#include "MenuBar.h"

MenuBar* MenuBar::menuBar = 0;

MenuBar::MenuBar(QWidget* parent) : QMenuBar(parent)
{
  createActions();
  createMenus();

  menuBar = this;
}

MenuBar::~MenuBar()
{
  menuBar = 0;
}

void MenuBar::createActions()
{

  fileMenu = new QMenu(tr("&File"), this);
  editMenu = new QMenu(tr("&Edit"), this);
  viewMenu = new QMenu(tr("&View"), this);
  simMenu = new QMenu(tr("&Simulation"), this);
  winMenu = new QMenu(tr("&Window"), this);
  helpMenu = new QMenu(tr("&Help"), this);
  surfaceMenu = new QMenu(tr("&Surface Rendering"), this);
  cameraMenu = new QMenu(tr("&Camera"), this);
  fovMenu = new QMenu(tr("&Vertical Opening Angle"), this);
  dragMenu = new QMenu(tr("&Drag and Drop"), this);
  resizeMenu = new QMenu(tr("&Resize"), this);

  // Surface submenu

  surfaceWire = new QAction(tr("Appearance &Wire Frame"), this);
  surfaceWire->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_W));
  surfaceWire->setCheckable(true);
  surfaceFlat = new QAction(tr("Appearance &Flat Shading"), this);
  surfaceFlat->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_F));
  surfaceFlat->setCheckable(true);
  surfaceSmooth = new QAction(tr("Appearance &Smooth Shading"), this);
  surfaceSmooth->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_M));
  surfaceSmooth->setCheckable(true);
  surfaceTexture = new QAction(tr("Appearance &Texture Shading"), this);
  surfaceTexture->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_T));
  surfaceTexture->setCheckable(true);
  surfaceCameraSim = new QAction(tr("Appearance &Camera Simulation"), this);
  surfaceCameraSim->setCheckable(true);

  surfacePhysicsWire = new QAction(tr("Physics &Wire Frame"), this);
  surfacePhysicsWire->setCheckable(true);
  surfacePhysicsFlat = new QAction(tr("Physics &Flat Shading"), this);
  surfacePhysicsFlat->setCheckable(true);
  surfacePhysicsSmooth = new QAction(tr("Physics &Smooth Shading"), this);
  surfacePhysicsSmooth->setCheckable(true);

  surfaceMixed1 = new QAction(tr("Mixed Mode &1 (Shaded Physics)"), this);
  surfaceMixed1->setCheckable(true);
  surfaceMixed2 = new QAction(tr("Mixed Mode &2 (Shaded Appearance)"), this);
  surfaceMixed2->setCheckable(true);

  surfaceActionGroup = new QActionGroup(this);
  surfaceActionGroup->addAction(surfaceWire);
  surfaceActionGroup->addAction(surfaceFlat);
  surfaceActionGroup->addAction(surfaceSmooth);
  surfaceActionGroup->addAction(surfaceTexture);
  surfaceActionGroup->addAction(surfaceCameraSim);
  surfaceActionGroup->addAction(surfacePhysicsWire);
  surfaceActionGroup->addAction(surfacePhysicsFlat);
  surfaceActionGroup->addAction(surfacePhysicsSmooth);
  surfaceActionGroup->addAction(surfaceMixed1);
  surfaceActionGroup->addAction(surfaceMixed2);

  // Camera submenu

  camTarget = new QAction(tr("&Target Mode"), this);
  camTarget->setCheckable(true);
  camFree = new QAction(tr("&Free Mode"), this);
  camFree->setCheckable(true);
  camPresentation = new QAction(tr("&Presentation Mode"), this);
  camPresentation->setCheckable(true);
  
  camReset = new QAction(tr("&Reset"), this);
  camReset->setShortcut(QKeySequence(Qt::Key_R));
  camToggle = new QAction(tr("&Toggle"), this);
  camToggle->setShortcut(QKeySequence(Qt::Key_T));
  camFit = new QAction(tr("&Fit"), this);
  camFit->setShortcut(QKeySequence(Qt::Key_F));

  camSnapToRoot = new QAction(tr("&Snap To Root"), this);
  camSnapToRoot->setCheckable(true);

  cameraActionGroup = new QActionGroup(this);
  cameraActionGroup->addAction(camTarget);
  cameraActionGroup->addAction(camFree);
  cameraActionGroup->addAction(camPresentation);

  // fov submenu

  fov20 = new QAction(tr("&20"), this);
  fov20->setShortcut(QKeySequence(Qt::Key_1));
  fov20->setCheckable(true);
  fov40 = new QAction(tr("&40"), this);
  fov40->setShortcut(QKeySequence(Qt::Key_2));
  fov40->setCheckable(true);
  fov60 = new QAction(tr("&60"), this);
  fov60->setShortcut(QKeySequence(Qt::Key_3));
  fov60->setCheckable(true);
  fov80 = new QAction(tr("&80"), this);
  fov80->setShortcut(QKeySequence(Qt::Key_4));
  fov80->setCheckable(true);
  fov100 = new QAction(tr("100"), this);
  fov100->setShortcut(QKeySequence(Qt::Key_5));
  fov100->setCheckable(true);
  fov120 = new QAction(tr("120"), this);
  fov120->setShortcut(QKeySequence(Qt::Key_6));
  fov120->setCheckable(true);

  fovActionGroup = new QActionGroup(this);
  fovActionGroup->addAction(fov20);
  fovActionGroup->addAction(fov40);
  fovActionGroup->addAction(fov60);
  fovActionGroup->addAction(fov80);
  fovActionGroup->addAction(fov100);
  fovActionGroup->addAction(fov120);

  // drag submenu

  dragXY = new QAction(tr("X/Y Plane"), this);
  dragXY->setShortcut(QKeySequence(Qt::Key_Z));
  dragXY->setCheckable(true);
  dragXZ = new QAction(tr("X/Z Plane"), this);
  dragXZ->setShortcut(QKeySequence(Qt::Key_Y));
  dragXZ->setCheckable(true);
  dragYZ = new QAction(tr("Y/Z Plane"), this);
  dragYZ->setShortcut(QKeySequence(Qt::Key_X));
  dragYZ->setCheckable(true);

  dragPlaneActionGroup = new QActionGroup(this);
  dragPlaneActionGroup->addAction(dragXY);
  dragPlaneActionGroup->addAction(dragXZ);
  dragPlaneActionGroup->addAction(dragYZ);

  dragKeepDynamics = new QAction(tr("&Keep Dynamics"), this);
  dragKeepDynamics->setShortcut(QKeySequence(Qt::Key_8));
  dragKeepDynamics->setCheckable(true);
  dragResetDynamics = new QAction(tr("&Reset Dynamics"), this);
  dragResetDynamics->setShortcut(QKeySequence(Qt::Key_9));
  dragResetDynamics->setCheckable(true);
  dragApplyDynamics = new QAction(tr("&Apply Dynamics"), this);
  dragApplyDynamics->setShortcut(QKeySequence(Qt::Key_0));
  dragApplyDynamics->setCheckable(true);

  dragModeActionGroup = new QActionGroup(this);
  dragModeActionGroup->addAction(dragKeepDynamics);
  dragModeActionGroup->addAction(dragResetDynamics);
  dragModeActionGroup->addAction(dragApplyDynamics);

  // resize sub menu

  resize160x120 = new QAction(tr("160 x 120"), this);
  resize320x240 = new QAction(tr("320 x 240"), this);
  resize480x360 = new QAction(tr("480 x 360"), this);
  resize640x480 = new QAction(tr("640 x 480"), this);

  // File

  fileNewAction = new QAction(QIcon(":/icons/New.png"), tr("&New"), this);
  fileNewAction->setShortcut(QKeySequence(QKeySequence::New));
  fileNewAction->setStatusTip(tr("Create a new simulation file"));
  fileNewAction->setEnabled(false);

  fileOpenAction = new QAction(QIcon(":/icons/Open.png"), tr("&Open..."), this); 
  fileOpenAction->setShortcut(QKeySequence(QKeySequence::Open));
  fileOpenAction->setStatusTip(tr("Open an existing simulation file"));
  fileOpenAction->setEnabled(false);

  fileSaveAction = new QAction(QIcon(":/icons/Save.png"), tr("&Save"), this);
  fileSaveAction->setShortcut(QKeySequence(QKeySequence::Save));
  fileSaveAction->setStatusTip(tr("Save the document to disk"));
  fileSaveAction->setEnabled(false);

  fileSaveAsAction = new QAction(tr("Save &As..."), this);
  fileSaveAsAction->setStatusTip(tr("Save the document under a new name"));
  fileSaveAsAction->setEnabled(false);

  fileCloseAction = new QAction(tr("&Close"), this);
  fileCloseAction->setStatusTip(tr("Close the document"));
  fileCloseAction->setEnabled(false);

  fileExitAction = new QAction(/*QIcon(":/icons/Exit.png"), */tr("E&xit"), this);
#ifdef _WIN32
  fileExitAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_F4, Qt::CTRL + Qt::Key_Q));
#else
  fileExitAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q, Qt::ALT + Qt::Key_F4));
#endif
  fileExitAction->setStatusTip(tr("Exit the application"));
  fileExitAction->setEnabled(false);

  // Edit

  editUndoAction = new QAction(QIcon(":/icons/Undo.png"), tr("&Undo"), this);
  editUndoAction->setShortcut(QKeySequence(QKeySequence::Undo));
  editUndoAction->setStatusTip(tr("Undo the last action"));
  editUndoAction->setEnabled(false);

  editRedoAction = new QAction(QIcon(":/icons/Redo.png"), tr("&Redo"), this);
  editRedoAction->setShortcut(QKeySequence(QKeySequence::Redo));
  editRedoAction->setStatusTip(tr("Redo the previously undone action"));
  editRedoAction->setEnabled(false);

  editCutAction = new QAction(QIcon(":/icons/Cut.png"), tr("Cu&t"), this);
  editCutAction->setShortcut(QKeySequence(QKeySequence::Cut));
  editCutAction->setStatusTip(tr("Cut the current selection's contents to the clipboard"));
  editCutAction->setEnabled(false);

  editCopyAction = new QAction(QIcon(":/icons/Copy.png"), tr("&Copy"), this);
  editCopyAction->setShortcut(QKeySequence(QKeySequence::Copy));
  editCopyAction->setStatusTip(tr("Copy the current selection's contents or view to the clipboard"));
  editCopyAction->setEnabled(false);

  editPasteAction = new QAction(QIcon(":/icons/Paste.png"), tr("&Paste"), this);
  editPasteAction->setShortcut(QKeySequence(QKeySequence::Paste));
  editPasteAction->setStatusTip(tr("Paste the clipboard's contents into the current selection"));
  editPasteAction->setEnabled(false);

  editPlaneAction = dragMenu->menuAction();
  editPlaneAction->setIcon(QIcon(":/icons/DragPlane.png"));
  editPlaneAction->setStatusTip(tr("Select the drag and drop dynamics mode and plane along which operations are performed"));
  editPlaneAction->setEnabled(false);

  // View

  viewToolBarAction = new QAction(tr("Tool&bar"), this);
  viewToolBarAction->setStatusTip(tr("Show or hide the toolbar"));
  viewToolBarAction->setEnabled(false);
  viewToolBarAction->setCheckable(true);

  viewStatusBarAction = new QAction(tr("&Status Bar"), this);
  viewStatusBarAction->setStatusTip(tr("Show or hide the status bar"));
  viewStatusBarAction->setEnabled(false);
  viewStatusBarAction->setCheckable(true);

  viewConsoleAction = new QAction(tr("&Console"), this);
  viewConsoleAction->setShortcut(QString("Alt+C"));
  viewConsoleAction->setStatusTip(tr("Show or hide the console window"));
  viewConsoleAction->setEnabled(false);
  viewConsoleAction->setCheckable(true);
  
  viewTreeAction = new QAction(QIcon(":/icons/Tree.png"), tr("&Tree View"), this);
  viewTreeAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_T));
  viewTreeAction->setStatusTip(tr("Show or hide the tree window"));
  viewTreeAction->setEnabled(false);
  viewTreeAction->setCheckable(true);

  viewEditorAction = new QAction(tr("&Editor"), this);
  viewEditorAction->setStatusTip(tr("Show or hide the document editor"));
  viewEditorAction->setEnabled(false);
  
  viewGridAction = new QAction(QIcon(":/icons/Grid.png"), tr("Show &Grid"), this);
  viewGridAction->setStatusTip(tr("Show a grid behind the view"));
  viewGridAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_G));
  viewGridAction->setCheckable(true);
  viewGridAction->setEnabled(false);

  viewPlaneAction = new QAction(tr("Show &Plane"), this);
  viewPlaneAction->setStatusTip(tr("Show the currently selected drag and drop plane"));
  viewPlaneAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_P));
  viewPlaneAction->setCheckable(true);
  viewPlaneAction->setEnabled(false);
  
  viewSurfaceAction = surfaceMenu->menuAction();
  viewSurfaceAction->setIcon(QIcon(":/icons/Surface.png"));
  viewSurfaceAction->setStatusTip(tr("Select different rendering techniques for displaying the scene"));
  viewSurfaceAction->setEnabled(false);
  
  viewCameraAction = cameraMenu->menuAction();
  viewCameraAction->setIcon(QIcon(":/icons/Camera.png"));
  viewCameraAction->setStatusTip(tr("Select different camera modes for displaying the scene"));
  viewCameraAction->setEnabled(false);
  
  viewDetailSensorsAction = new QAction(QIcon(":/icons/Sensor.png"), tr("Show &Sensors"), this);
  viewDetailSensorsAction->setStatusTip(tr("Show the values of the sensors in the scene view"));
  viewDetailSensorsAction->setCheckable(true);
  viewDetailSensorsAction->setEnabled(false);

  viewControllerDrawingsAction = new QAction(tr("Show Controller &Drawings"), this);
  viewControllerDrawingsAction->setStatusTip(tr("Show the controller 3D drawings in the scene view"));
  viewControllerDrawingsAction->setCheckable(true);
  viewControllerDrawingsAction->setEnabled(false);
  
  viewFovAction = fovMenu->menuAction();
  viewFovAction->setEnabled(false);
  
  // Simulation

  simResetAction = new QAction(QIcon(":/icons/Reset.png"), tr("&Reset"), this);
  simResetAction->setStatusTip(tr("Reset the simulation to the beginning"));
  simResetAction->setShortcut(QKeySequence(Qt::SHIFT + Qt::Key_F5));
  simResetAction->setEnabled(false);

  simStartAction = new QAction(QIcon(":/icons/Start.png"), tr("&Start"), this);
  simStartAction->setStatusTip(tr("Start or stop the simulation"));
  simStartAction->setShortcut(QKeySequence(Qt::Key_F5));
  simStartAction->setEnabled(false);
  simStartAction->setCheckable(true);

  simStepAction = new QAction(QIcon(":/icons/Step.png"), tr("&Step"), this);
  simStepAction->setStatusTip(tr("Execute a single simulation step"));
  simStepAction->setShortcut(QKeySequence(Qt::Key_F8));
  simStepAction->setEnabled(false);

  simGenerateCubeMapAction = new QAction(tr("Generate &Cube Map"), this);
  simGenerateCubeMapAction->setStatusTip(tr("Generate a cube map and save it to scene path"));
  simGenerateCubeMapAction->setEnabled(false);

  simShowFpsAction = new QAction(tr("Show &FPS"), this);
  simShowFpsAction->setStatusTip(tr("Show the frames per second per frame buffer as object window message"));
  simShowFpsAction->setEnabled(false);

  simShowStatistics = new QAction(tr("Show &Statistics"), this);
  simShowStatistics->setStatusTip(tr("Show the polygon count of scene and selected object as object window message"));
  simShowStatistics->setEnabled(false);

  // Window

  winCloseAction = new QAction(tr("Cl&ose"), this);
  winCloseAction->setShortcut(QKeySequence(QKeySequence::Close));
  winCloseAction->setStatusTip(tr("Close the active window"));
  winCloseAction->setEnabled(false);

  winCloseAllAction = new QAction(tr("Close &All"), this);
#ifdef _WIN32
  winCloseAllAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_F4));
#else
  winCloseAllAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_W));
#endif
  winCloseAllAction->setStatusTip(tr("Close all the windows"));
  winCloseAllAction->setEnabled(false);

  winTileAction = new QAction(QIcon(":/icons/Tile.png"), tr("&Tile"), this);
  winTileAction->setStatusTip(tr("Tile the windows"));
  winTileAction->setEnabled(false);

  winCascadeAction = new QAction(QIcon(":/icons/Cascade.png"), tr("&Cascade"), this);
  winCascadeAction->setStatusTip(tr("Cascade the windows"));
  winCascadeAction->setEnabled(false);
  
  winNextAction = new QAction(QIcon(":/icons/Next.png"), tr("Ne&xt"), this);
  winNextAction->setShortcut(QKeySequence(QKeySequence::NextChild));
  winNextAction->setStatusTip(tr("Move the focus to the next window"));
  winNextAction->setEnabled(false);

  winPreviousAction = new QAction(QIcon(":/icons/Previous.png"), tr("Pre&vious"), this);
  winPreviousAction->setShortcut(QKeySequence(QKeySequence::PreviousChild));
  winPreviousAction->setStatusTip(tr("Move the focus to the previous window"));
  winPreviousAction->setEnabled(false);

  // Help
  
  helpAboutAction = new QAction(tr("&About..."), this);
  helpAboutAction->setStatusTip(tr("Show the application's About box"));
  helpAboutAction->setEnabled(false);
  
  helpAboutQtAction = new QAction(tr("About &Qt..."), this);
  helpAboutQtAction->setStatusTip(tr("Show the Qt library's About box"));
  helpAboutQtAction->setEnabled(false);

  helpContextAction = new QAction(QIcon(":/icons/Help.png"), tr("&Help"), this);
  helpContextAction->setStatusTip(tr("Display help for clicked on buttons, menus and windows"));
  helpContextAction->setEnabled(false);

}

void MenuBar::createMenus()
{
  resetFileMenu();
  addMenu(fileMenu);

  resetEditMenu();
  addMenu(editMenu);

  resetViewMenu();
  addMenu(viewMenu);

  simMenu->addAction(simResetAction);
  simMenu->addAction(simStartAction);
  simMenu->addAction(simStepAction);
  simMenu->addSeparator();
  simMenu->addAction(simGenerateCubeMapAction);
  simMenu->addAction(simShowFpsAction);
  simMenu->addAction(simShowStatistics);
  addMenu(simMenu);

  resetWinMenu();
  addMenu(winMenu);

  addSeparator();

  helpMenu->addAction(helpAboutAction);
  helpMenu->addAction(helpAboutQtAction);
  addMenu(helpMenu);

  surfaceMenu->addAction(surfaceWire);
  surfaceMenu->addAction(surfaceFlat);
  surfaceMenu->addAction(surfaceSmooth);
  surfaceMenu->addAction(surfaceTexture);
  surfaceMenu->addAction(surfaceCameraSim);
  surfaceMenu->addSeparator();
  surfaceMenu->addAction(surfacePhysicsWire);
  surfaceMenu->addAction(surfacePhysicsFlat);
  surfaceMenu->addAction(surfacePhysicsSmooth);
  surfaceMenu->addSeparator();
  surfaceMenu->addAction(surfaceMixed1);
  surfaceMenu->addAction(surfaceMixed2);

  cameraMenu->addAction(camTarget);
  cameraMenu->addAction(camFree);
  cameraMenu->addAction(camPresentation);
  cameraMenu->addSeparator();
  cameraMenu->addAction(camReset);
  cameraMenu->addAction(camToggle);
  cameraMenu->addAction(camFit);
  cameraMenu->addSeparator();
  cameraMenu->addAction(camSnapToRoot);

  fovMenu->addAction(fov20);
  fovMenu->addAction(fov40);
  fovMenu->addAction(fov60);
  fovMenu->addAction(fov80);
  fovMenu->addAction(fov100);
  fovMenu->addAction(fov120);

  dragMenu->addAction(dragXY);
  dragMenu->addAction(dragXZ);
  dragMenu->addAction(dragYZ);
  dragMenu->addSeparator();
  dragMenu->addAction(dragKeepDynamics);
  dragMenu->addAction(dragResetDynamics);
  dragMenu->addAction(dragApplyDynamics);

  resizeMenu->addAction(resize160x120);
  resizeMenu->addAction(resize320x240);
  resizeMenu->addAction(resize480x360);
  resizeMenu->addAction(resize640x480);
}

void MenuBar::resetFileMenu()
{
  fileMenu->clear();
  fileMenu->addAction(fileNewAction);
  fileMenu->addAction(fileOpenAction);
  fileMenu->addSeparator();
  fileMenu->addAction(fileSaveAction);
  fileMenu->addAction(fileSaveAsAction);
  fileMenu->addSeparator();
  fileMenu->addAction(fileCloseAction);
}

void MenuBar::resetWinMenu()
{
  winMenu->clear();
  winMenu->addAction(winCloseAction);
  winMenu->addAction(winCloseAllAction);
  winMenu->addSeparator();
  winMenu->addAction(winTileAction);
  winMenu->addAction(winCascadeAction);
  winMenu->addSeparator();
  winMenu->addAction(winNextAction);
  winMenu->addAction(winPreviousAction);
  winMenu->addSeparator();
  winMenu->addMenu(resizeMenu);
}

void MenuBar::resetEditMenu()
{
  editMenu->clear();
}

void MenuBar::resetViewMenu()
{
  viewMenu->clear();
  viewMenu->addAction(viewToolBarAction);
  viewMenu->addAction(viewStatusBarAction);
  viewMenu->addAction(viewConsoleAction);
  viewMenu->addAction(viewTreeAction);
  viewMenu->addSeparator();
  viewMenu->addAction(viewEditorAction);
}
