
#ifndef ObjectWindow_H
#define ObjectWindow_H

#include <GL/glew.h>
#include <QGLWidget>
#include <QSignalMapper>

#include "SubWindow.h"
#include "../SimRobotCore/OpenGL/ObjectRenderer.h"

class ObjectGLWidget;

class ObjectWindow : public SubWindow
{
  Q_OBJECT

public:
  ObjectWindow(QMdiArea* mdiArea, const QString& object);

private slots:
  void setSurfaceStyleSlot(int style);
  void setCameraModeSlot(int mode);
  void setFovYSlot(int fovy);
  void setDragPlaneSlot(int plane);
  void setDragModeSlot(int mode);
  void resetCameraSlot();
  void toggleCameraModeSlot();
  void fitCameraSlot();
  void snapToRootSlot();
  void showDetailSensorsSlot();
  void showControllerDrawingsSlot();
  void copySlot();

private:
  void connectMenuBar();
  void disconnectMenuBar();
  void restoreLayout();
  void writeLayout();
  void updateObject();

  void keyPressEvent(QKeyEvent* event);
  void wheelEvent(QWheelEvent* event);
  void focusInEvent(QFocusEvent* event);
  void focusOutEvent(QFocusEvent* event);

  ObjectRenderer objectRenderer;
  ObjectGLWidget* objectGlWidget;
  QSignalMapper surfaceMapper;
  QSignalMapper cameraMapper;
  QSignalMapper fovMapper;
  QSignalMapper dragPlaneMapper;
  QSignalMapper dragModeMapper;
};

class ObjectGLWidget : public QGLWidget
{

public:
  ObjectGLWidget(ObjectRenderer& renderer, ObjectWindow* parent);
  ~ObjectGLWidget();

private:
  void initializeGL();
  void paintGL();
  void resizeGL(int width, int height);
  void focusInEvent(QFocusEvent* event);
  void focusOutEvent(QFocusEvent* event);
  void mouseMoveEvent(QMouseEvent* event);
  void mousePressEvent(QMouseEvent* event);
  void mouseReleaseEvent(QMouseEvent* event);
  void contextMenuEvent(QContextMenuEvent* event);

  static QList<ObjectGLWidget*> objectGLWidgetList;

  ObjectRenderer& objectRenderer;
  ObjectWindow* parent;
};

#endif // ObjectWindow_H
