
#include <QSizeGrip>
#include <QLabel>

#include "StatusBar.h"
#include "MenuBar.h"
#include "Document.h"
#include "../SimRobotCore/Simulation/Simulation.h"

StatusBar::StatusBar(QWidget* parent) : QStatusBar(parent), lastGotSimulation(false)
{
  collisionsLabel = new QLabel(this);
  collisionsLabel->setMinimumWidth(50);
  collisionsLabel->setEnabled(false);
  frameRateLabel = new QLabel(this);
  frameRateLabel->setMinimumWidth(50);
  frameRateLabel->setEnabled(false);
  stepsLabel = new QLabel(this);
  stepsLabel->setMinimumWidth(50);
  stepsLabel->setEnabled(false);

  setSizeGripEnabled(false);

  addPermanentWidget(collisionsLabel);
  addPermanentWidget(frameRateLabel);
  addPermanentWidget(stepsLabel);

  addPermanentWidget(new QSizeGrip(this));

  showMessage("Ready");

  connect(MenuBar::menuBar->viewStatusBarAction, SIGNAL(triggered(bool)), SLOT(setVisible(bool)));
  MenuBar::menuBar->
  MenuBar::menuBar->viewStatusBarAction->setEnabled(true);

  connect(Document::document, SIGNAL(updateStatusBarSignal()), SLOT(updateSlot()));
}

void StatusBar::hideEvent(QHideEvent* event)
{
  QStatusBar::hideEvent(event);

  MenuBar::menuBar->viewStatusBarAction->setChecked(false);
}

void StatusBar::showEvent(QShowEvent * event)
{
  QStatusBar::showEvent(event);

  MenuBar::menuBar->viewStatusBarAction->setChecked(true);
}

void StatusBar::updateSlot()
{
  const QString& newMessage(Document::document->getStatusMessage());
  const QString& curMessage(currentMessage());
  if(curMessage.isEmpty() || curMessage == lastStatusMessage)
    showMessage(newMessage);
  lastStatusMessage = newMessage;

  Simulation* simulation = Document::document->getSimulation();
  bool gotSimulation = simulation ? true : false;
  if(gotSimulation != lastGotSimulation)
  {
    lastGotSimulation = gotSimulation;
    collisionsLabel->setEnabled(gotSimulation);
    frameRateLabel->setEnabled(gotSimulation);
    stepsLabel->setEnabled(gotSimulation);
  }
  if(simulation)
  {
    char buf[20];
    int val;

    sprintf(buf, "%03d", simulation->getNumberOfCollisions());
    collisionsLabel->setText(buf);

    val = simulation->getFrameRate();
    if(val > 999)
      val = 999;
    sprintf(buf, "%03d", val);
    frameRateLabel->setText(buf);

    sprintf(buf, "%06d", simulation->getSimulationStep());
    stepsLabel->setText(buf);
  }
  else
  {    
    collisionsLabel->setText("000");
    frameRateLabel->setText("000");
    stepsLabel->setText("000000");
  }
}
