
#include <QMdiArea>
#include <QCloseEvent>
#include <QLayout>

#include "SubWindow.h"
#include "Document.h"

SubWindow::SubWindow(QMdiArea* mdiArea, const QString& object) : QMdiSubWindow(mdiArea),
  object(object), layoutWritten(false), layoutRestored(false) {}

void SubWindow::show()
{
  if(!layoutRestored)
  {
    layoutRestored = true;
    restoreLayout();
  }
  QMdiSubWindow::show();
}

bool SubWindow::close()
{
  if(layoutWritten)
    return false;
  layoutWritten = true;
  writeLayout();
  return QMdiSubWindow::close();
}

void SubWindow::closeEvent(QCloseEvent* event)
{
  if(!layoutWritten)
  {
    layoutWritten = true;
    writeLayout();
    Document::document->closeObject(object);
  }
  QMdiSubWindow::closeEvent(event);
}

const QString& SubWindow::getObject() const
{
  return object;
}

void SubWindow::writeGeometry(QSettings& settings)
{
  settings.setValue("Geometry", normalGeometry);
  settings.setValue("WindowState", int(windowState()));
}

void SubWindow::restoreGeometry(QSettings& settings)
{
  const QVariant& var(settings.value("Geometry"));
  if(!var.isValid())
    return;
  const QRect& geometry(var.toRect());
  if(geometry.isEmpty())
    return;
  Qt::WindowStates newWindowState = Qt::WindowStates(settings.value("WindowState").toInt());

  Q_ASSERT(!(windowState() & (Qt::WindowMinimized | Qt::WindowMaximized | Qt::WindowFullScreen)));
  setGeometry(geometry);
  setWindowState(newWindowState);

  if(!(newWindowState & (Qt::WindowMinimized | Qt::WindowMaximized | Qt::WindowFullScreen)))
    setGeometry(geometry); // bugfix!?
  normalGeometry = geometry;
}

void SubWindow::moveEvent(QMoveEvent* event)
{
  if(!(windowState() & (Qt::WindowMinimized | Qt::WindowMaximized | Qt::WindowFullScreen)))
    normalGeometry = geometry();
  
  QMdiSubWindow::moveEvent(event);
}

void SubWindow::resizeEvent(QResizeEvent* event)
{
  if(!(windowState() & (Qt::WindowMinimized | Qt::WindowMaximized | Qt::WindowFullScreen)))
    normalGeometry = geometry();

  QMdiSubWindow::resizeEvent(event);
}

void SubWindow::setWidget(QWidget* widget)
{
  if(widget && isVisible())
  { // this avois unwanted "to foreground jumps" of this sub window
    widget->setParent(this);
    widget->resize(0, 0);
    widget->setVisible(true);
  }
  QMdiSubWindow::setWidget(widget);
}
