-- version identifier
global g_version = "B-Human 3DSMax Exporter V0.4"
global g_date = "2008-01-24"
global g_author = "Dennis Pachur"
global g_contact = "dpachur@gmx.de"
global g_howto = "1. select objects\n2. convert them to editable meshes\n3. clear smgs\n4. set up smoothing groups for round shaded objects\n5. check export options\n6. export"
global g_help = "\nreverse normals:\nreverse the face normal.\n
\nflip vertical tex coords:\nreverse the vertical texture coordinates.\n
\ncombine primitives:\nif checked a text string will be placed as loading attribute of SimRobot in order to combine primitives instead of rendering every triangle for its own\n
\nautomatic filenames:\ngenerate filenames according to the objects names and place them into the 3dsMax_Export folder at the last set folder (if nothing has set -> desktop)\n
\nexport to one file:\nexport everything to one file\n
\nuse multi-submaterials:\nsplit the meshes by their material ids\n"


-- global variables
global g_reversenormals = true
global g_flipverticaltexcoord = true
global g_combine_primitives = true
global g_automatic_filename = false
global g_export_to_one_file = false
global g_use_multimaterials = true
global g_printheader = true
global g_printfooter = true

--maxscript doesn't provide a way to format float output: here it is
fn printfloat flt =
(
	local numdigits = 10
	(
		str = ""
		if flt<0 then str += "-"
		flt2 = abs flt
		str += ((flt2 as integer) as string)
		flt2 -= (flt2 as integer)
		str +="."
		for i=1 to numdigits do
		(
			flt2 *=10.0
			str += ((flt2 as integer) as string)
			flt2 -= (flt2 as integer)
		)
		return str
	)
)

-- definition of a triangle
struct triangle
(
	v1,
	v2,
	v3,
	
	fn tofile offset =
	(
		str = stringstream ""
		if g_reversenormals == false then
			format "v1=\"%\" v2=\"%\" v3=\"%\"" (v1+offset) (v3+offset) (v2+offset) to:str
		else
			format "v1=\"%\" v2=\"%\" v3=\"%\"" (v1+offset) (v2+offset) (v3+offset) to:str
		return str
	)
)

-- definition of a vertex
struct vertex
(
	pos,
	vertindex,
	tvertindex,
	uvmap,
	
	fn tofile =
	(
		str = stringstream ""
		if g_flipverticaltexcoord and (uvmap != undefined) then uvmap[2]=1.0-uvmap[2]
		if(uvmap == undefinied) then
			format "x=\"%\" y=\"%\" z=\"%\"" (printfloat pos[1]) (printfloat pos[2]) (printfloat pos[3]) to:str
		else
			format "x=\"%\" y=\"%\" z=\"%\" s=\"%\" t=\"%\"" (printfloat pos[1]) (printfloat pos[2]) (printfloat pos[3]) (printfloat uvmap[1]) (printfloat uvmap[2]) to:str
		if g_flipverticaltexcoord and (uvmap != undefined) then uvmap[2]=1.0-uvmap[2]
		return str
	)
)

-- definiton of a complex shape container
struct complexshape
(
	vertices = #(),
	triangles = #(),
	name,
	id,
	offset,
	hastexture = false,
	texturename,
	
	fn tofile =
	(
		sstr = stringstream "" 
		
		-- print header
		format "  <Macro name=\"%\">\n" ((name as string) + "-" + (id as string)) to:sstr
		format "    <ComplexShape name=\"%\">\n" ((name as string) + "-CS" + (id as string)) to:sstr
		
		-- print appearance definition
		if(hastexture) do
			format "      <Appearance ref=\"%\"/>\n" (texturename as string) to:sstr
		
		-- print triangle information
		format "      <GraphicalRepresentation vertexList=\"%\">\n" ((name as string) + ".vertices") to:sstr
		for t=1 to triangles.count do
		(
			ExporterRollout.pjob_writedata.value = ((t*50)/triangles.count + 50)
			format "        <Triangle %/>\n" (triangles[t].tofile(offset) as string) to:sstr
		)
		ExporterRollout.pjob_writedata.color = [0, 200, 40]
		format "        <GraphicalAttributes>\n" to:sstr
		if id > 1 then
			format "          <SmoothSurface value=\"true\"/>\n" to:sstr
		else
			format "          <SmoothSurface value=\"false\"/>\n" to:sstr
		if g_combine_primitives then
			format "          <CombinePrimitives value=\"true\"/>\n" to:sstr
		else
			format "          <CombinePrimitives value=\"false\"/>\n" to:sstr
		format "        </GraphicalAttributes>\n" to:sstr
		format "      </GraphicalRepresentation>\n" to:sstr
		
		-- print footer
		format "      <PhysicalRepresentation/>\n" to:sstr
		format "    </ComplexShape>\n" to:sstr
		format "  </Macro>\n" to:sstr
				
		return sstr
	),
		
	fn addtriangle obj triindx self =
	(
		vertindices = getface obj triindx
		if (meshop.getMapSupport obj 1) then
			tvertindices = meshop.getmapface obj 1 triindx
		else
			tvertindices = [0,0,0]
		vertindices = #(vertindices.x as integer, vertindices.y as integer, vertindices.z as integer)
		tvertindices = #(tvertindices.x as integer, tvertindices.y as integer, tvertindices.z as integer)
		CSvertindices = #()
		for i=1 to 3 do
		(
			CSvertindices[i] = undefined
			for v=1 to self.vertices.count do
			(
				if (vertindices[i] == self.vertices[v].vertindex and tvertindices[i] == self.vertices[v].tvertindex) do
				(
					CSvertindices[i] = v
				)
			)
			if (CSvertindices[i] == undefined) do
			(
				pos = obj.verts[vertindices[i]].pos
				if (meshop.getMapSupport obj 1) then
				(
					uv = meshop.getmapvert obj 1 tvertindices[i]
					newvert = vertex pos vertindices[i] tvertindices[i] #(uv.x, uv.y)
					self.hastexture = true
				)
				else
					newvert = vertex pos vertindices[i] tvertindices[i] undefined
				append self.vertices newvert
				CSvertindices[i] = self.vertices.count
			)
		)
		append self.triangles (triangle CSvertindices[1] CSvertindices[2] CSvertindices[3])
	)
)


struct multisubobject
(
	cshapes = #(),
	
	fn init =
	(
		-- create maximum number of possible complex shapes (10)
		for cs=1 to 10 do
			cshapes[cs] = complexshape()
	)
)

fn clear_smgs =
(
	-- get selection a check some conditions
	cursel = GetCurrentSelection()
	for current_selected_mesh=1 to cursel.count do
	(
		mesh = cursel[current_selected_mesh]
		if mesh == undefined do
		(
			messagebox "no object selected"
			return undefined
		)
		for currentFace=1 to mesh.numfaces do
			setFaceSmoothGroup mesh currentFace 0
	)
	messagebox "Done"
)

fn exportobj obj = 
(
	-- set locals
	max modify mode
	set coordsys world
	local multisubobjects = #()
	
	-- scan model for multisubobjects
	faces = obj.faces
	usedmaterials = #()
	if(g_use_multimaterials == true) then
	(
		for f=1 to faces.count do
		(
			m = getfacematid obj f
			j=1 
			found=false
			while (j<=usedmaterials.count and found==false) do
			(
				if usedmaterials[j]==m then found=true
				j+=1
			)
			if (found==false) do
				usedmaterials[usedmaterials.count+1]=m
		)
		for mso=1 to usedmaterials.count do
		(
			multisubobjects[usedmaterials[mso]] = multisubobject()
			multisubobjects[usedmaterials[mso]].init()
		)
	)
	else
	(
		multisubobjects[1] = multisubobject()
		multisubobjects[1].init()
	)
	
	-- gather information
	for f=1 to faces.count do
	(
		ExporterRollout.pjob_gatherdata.value = (f*100)/faces.count
		smgroup = (getFaceSmoothGroup obj f)+1
		if(g_use_multimaterials == false) then
		(
			multisubobjects[1].cshapes[smgroup].addtriangle obj f multisubobjects[1].cshapes[smgroup]
		)
		else
		(
			multiobjid = getfacematid obj f
			multisubobjects[multiobjid].cshapes[smgroup].addtriangle obj f multisubobjects[multiobjid].cshapes[smgroup]
		)
	)
	
	-- correct texture names
	if(g_use_multimaterials == true) then
	(
		for mso=1 to usedmaterials.count do
		(
			for cs=1 to multisubobjects[usedmaterials[mso]].cshapes.count do
				multisubobjects[usedmaterials[mso]].cshapes[cs].texturename = obj.material.materiallist[usedmaterials[mso]].name
		)
	)
	else
	(
		for cs=1 to multisubobjects[1].cshapes.count do
			multisubobjects[1].cshapes[cs].texturename = obj.material.name
	)
	ExporterRollout.pjob_gatherdata.color = [0, 200, 40]
		
	s = stringstream ""
	
	-- print header
	if(g_printheader) do
	(
		format "<RoSiIncludeFile>\n" to:s
		format "  <!-- object \"%\" -->\n"  (obj.name as string) to:s
		format "  <!-- code has been generated: % -->\n\n" (g_version as string) to:s
	)
		
	-- print vertex information
	for mso=1 to usedmaterials.count do
	(
		local vertexlistnotex = false
		local vertexlisttex = false
		local vertcount = 0
		for cs=1 to multisubobjects[usedmaterials[mso]].cshapes.count do
		(
			multisubobjects[usedmaterials[mso]].cshapes[cs].offset = vertcount
			vertcount += multisubobjects[usedmaterials[mso]].cshapes[cs].vertices.count
			if multisubobjects[usedmaterials[mso]].cshapes[cs].hastexture == true then
				vertexlisttex = true
			else if (multisubobjects[usedmaterials[mso]].cshapes[cs].vertices.count > 0) do
				vertexlistnotex = true
		)
		local currentvert = 0
		if vertexlistnotex == true and vertexlisttex == false then
		(
			format "  <VertexList name=\"%\">\n" ((obj.name + "-" + (mso as string) as string) + ".vertices") to:s
			for cs=1 to multisubobjects[usedmaterials[mso]].cshapes.count do
			(
				for v=1 to multisubobjects[usedmaterials[mso]].cshapes[cs].vertices.count do
				(
					currentvert += 1
					ExporterRollout.pjob_writedata.value = (50*currentvert)/vertcount
					format "    <Vertex name=\"%\" %/>\n" ((v + multisubobjects[usedmaterials[mso]].cshapes[cs].offset) as string) (multisubobjects[usedmaterials[mso]].cshapes[cs].vertices[v].tofile() as string) to:s
				)
			)
			format "  </VertexList>\n\n" to:s
		)
		else if vertexlistnotex == false and vertexlisttex == true then
		(
			format "  <VertexListWithTexCoords name=\"%\">\n" ((obj.name + "-" + (mso as string) as string) + ".vertices") to:s
			for cs=1 to multisubobjects[usedmaterials[mso]].cshapes.count do
			(
				for v=1 to multisubobjects[usedmaterials[mso]].cshapes[cs].vertices.count do
				(
					currentvert += 1
					ExporterRollout.pjob_writedata.value = (50*currentvert)/vertcount
					format "    <Vertex name=\"%\" %/>\n" ((v + multisubobjects[usedmaterials[mso]].cshapes[cs].offset) as string) (multisubobjects[usedmaterials[mso]].cshapes[cs].vertices[v].tofile() as string) to:s
				)
			)
			format "  </VertexListWithTexCoords>\n\n" to:s
		)
		else
		(
			format "todo...\n" to:s
		)
		
		-- print complex shapes informations
		for cs=1 to multisubobjects[usedmaterials[mso]].cshapes.count do
		(
			if(multisubobjects[usedmaterials[mso]].cshapes[cs].vertices.count > 0) do
			(
				multisubobjects[usedmaterials[mso]].cshapes[cs].name = (obj.name  + "-" + (mso as string))
				multisubobjects[usedmaterials[mso]].cshapes[cs].id = cs
				format "%\n" (multisubobjects[usedmaterials[mso]].cshapes[cs].tofile() as string) to:s
			)
		)
	)
	
	-- print footer
	if(g_printfooter) do
		format "</RoSiIncludeFile>" to:s
	
	return s
)

fn export =
(
	-- get selection a check some conditions
	cursel = GetCurrentSelection()
	if(g_export_to_one_file) then
	(
		outfilename = getsavefilename caption:"Save RoSi Include File" types:"RoSiIncludeFile (*.rsi)|*.rsi|No Suffix (*)|*" filename:"binder.rsi"
		if (outfilename != undefined) do
		(
			outfile = createfile outfilename
			for current_selected_mesh=1 to cursel.count do
			(
				if(current_selected_mesh == 1) then
					g_printheader = true
				else
					g_printheader = false
					
				if(current_selected_mesh == cursel.count) then
					g_printfooter = true
				else
					g_printfooter = false

				mesh = cursel[current_selected_mesh]
		
				if mesh == undefined do
				(
					messagebox "no object selected"
					return undefined
				)
				if (((classof mesh.baseobject) as string) != "Editable_mesh") do
				(
					messagebox "object has to be a editable_mesh"
					return undefined
				)
				
				filestring = exportobj mesh
				format "%" (filestring as string) to:outfile
				flush outfile
				ExporterRollout.pjob_writefile.value = current_selected_mesh*100/cursel.count
			)
			close outfile
		)
	)
	else
	(
		for current_selected_mesh = 1 to cursel.count do
		(
			mesh = cursel[current_selected_mesh]
	
			if mesh == undefined do
			(
				messagebox "no object selected"
				return undefined
			)
			if (((classof mesh.baseobject) as string) != "Editable_mesh") do
			(
				messagebox "object has to be a editable_mesh"
				return undefined
			)
			
			-- get file to export to and start job
			outfilename = ""
			if(g_automatic_filename) then
				outfilename = ".\\3dsMax_Export\\" + mesh.name + ".rsi"
			else
				outfilename = getsavefilename caption:"Save RoSi Include File" types:"RoSiIncludeFile (*.rsi)|*.rsi|No Suffix (*)|*" filename:(mesh.name +".rsi")
			if (outfilename != undefined) do
			(
				outfile = createfile outfilename
				if (outfile!= undefined) do
				(
					filestring = exportobj mesh
					format "%" (filestring as string) to:outfile
					flush outfile
					close outfile	
				)
			)
			ExporterRollout.pjob_writefile.value = current_selected_mesh*100/cursel.count
	
		)
	)
	ExporterRollout.pjob_writefile.color = [0, 200, 40]
	-- reset progress bars after 1 second
	ExporterRollout.clock.active = true
)

rollout ExporterRollout "Export" width:340 height:220
(	
	GroupBox grp1 "Export Options" pos:[10,10] width:150 height:144
	
	checkbox reversefacenormals "Checkbox" pos:[130,30] width:16 height:16 checked:true
	label label1 "reverse face-normals" pos:[15,30] width:112 height:16
	
	checkbox flipverticaltexcoords "Checkbox" pos:[130,50] width:16 height:16 checked:true
	label label2 "flip vertical texcoords" pos:[15,50] width:112 height:16

	checkbox combineprimitives "Checkbox" pos:[130,70] width:16 height:16 checked:true
	label label3 "combine primitives" pos:[15,70] width:112 height:16
	
	checkbox automaticfilename "Checkbox" pos:[130,90] width:16 height:16 checked:false
	label label4 "automatic filenames" pos:[15,90] width:112 height:16
	
	checkbox exporttoonefile "Checkbox" pos:[130,110] width:16 height:16 checked:false
	label label5 "export to one file" pos:[15,110] width:112 height:16
	
	checkbox usemultimaterials "Checkbox" pos:[130,130] width:16 height:16 checked:true
	label label6 "use multi-submaterial" pos:[15,130] width:112 height:16
	
	button export_button "Export" pos:[10,160] width:50 height:24 toolTip:"Export the currently selected object"
	button clear_smgs_button "Clear Smoothing Groups" pos:[70,160] width:120 height:24 toolTip:"Clear smoothing groups of all selected objects."
	button info_button "HowTo" pos:[270,160] width:50 height:24 toolTip:"How to use this exporter."
	button help_button "Help" pos:[210,160] width:50 height:24 toolTip:"Info about the export parameter."
	
	GroupBox grp2 "Progress" pos:[170,10] width:150 height:144
	
	label label21 "gathering information:" pos:[180, 30]
	progressbar pjob_gatherdata color:[200,30,30] value:0 pos:[180,50] width:130 height:10
	
	label label22 "generating data" pos:[180, 70]
	progressbar pjob_writedata color:[200,30,30] value:0 pos:[180,90] width:130 height:10
	
	label label23 "writing files" pos:[180, 110]
	progressbar pjob_writefile color:[200,30,30] value:0 pos:[180,130] width:130 height:10
	
	timer clock "reseter" interval:1000 active:false
	
	on clock tick do
	(
		-- reset progress bars
		ExporterRollout.pjob_gatherdata.color = [200,30,30]
		ExporterRollout.pjob_gatherdata.value = 0
		ExporterRollout.pjob_writedata.color = [200,30,30]
		ExporterRollout.pjob_writedata.value = 0
		ExporterRollout.pjob_writefile.color = [200,30,30]
		ExporterRollout.pjob_writefile.value = 0
		clock.active = false
	)

	on export_button pressed do
	(
		g_reversenormals = reversefacenormals.checked
		g_flipverticaltexcoord = flipverticaltexcoords.checked
		g_combine_primitives = combineprimitives.checked
		g_automatic_filename = automaticfilename.checked
		g_export_to_one_file = exporttoonefile.checked
		g_use_multimaterials = usemultimaterials.checked
		g_printheader = true
		g_printfooter = true
		if(g_automatic_filename) do
			makedir ".\\3dsMax_Export"
		export()
	)

	on info_button pressed do
	(
		str = stringstream ""
		format "% - %\n%, (%)\n\nUsage:\n%" g_version g_date g_author g_contact g_howto to:str
		messagebox (str as string)
	)
	
	on help_button pressed do
	(
		str = stringstream ""
		format "% - %\n%, (%)\n\nParamter Information:\n%" g_version g_date g_author g_contact g_help to:str
		messagebox (str as string)
	)
	
	on clear_smgs_button pressed do
	(
		clear_smgs()
	)
)

-- create the rollout window and add the rollout
if ExportFloater != undefined do
(
	closerolloutfloater ExportFloater
)

ExportFloater = newRolloutFloater g_version 340 220
addRollout ExporterRollout ExportFloater
