/** 
* @file ReleaseOptions.h
* This file declares a class representing the debugging options in release code.
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#ifndef __ReleaseOptions_h_
#define __ReleaseOptions_h_

#include "Tools/Streams/Streamable.h"

/**
* @class ReleaseOptions
* A class representing the debugging options in release code.
*/
class ReleaseOptions : public Streamable
{
private:
  virtual void serialize(In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN();
    STREAM(stopwatches);
    STREAM(sensorData);
    STREAM(robotHealth);
    STREAM(motionRequest);
    STREAM_REGISTER_FINISH();
  }

public:
  /**
  * Default constructor.
  */
  ReleaseOptions() : 
    stopwatches(false),
    sensorData(false),
    robotHealth(true),     // Send health data per default
    motionRequest(true) {} // same here

  bool stopwatches,   /**< Activate time measurements. */
       sensorData,    /**< Activate sending sensorData. */
       robotHealth,   /**< Activate sending robot health data. */
       motionRequest; /**< Activate sending motion requests. */
};

#endif // __ReleaseOptions_h_
