/** 
* @file Stopwatch.h
* The file declares the stopwatch macros.
* @author <a href="mailto:juengel@informatik.hu-berlin.de">Matthias Jngel</a>
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</a>
*/

#ifndef __Stopwatch_h_
#define __Stopwatch_h_

#include "Platform/SystemCall.h"

#ifdef RELEASE
#include "Tools/Team.h"
#include "Tools/Debugging/ReleaseOptions.h"

#define STOP_TIME_ON_REQUEST(eventID, expression) \
  if(Global::getReleaseOptions().stopwatches) \
  { \
    unsigned _startTimeN; \
    static unsigned _counter = 0; \
    _startTimeN = SystemCall::getRealSystemTime();\
    { expression }\
    TEAM_OUTPUT(idStopwatch, bin, eventID << _startTimeN << SystemCall::getRealSystemTime() << _counter++); \
  } \
  else \
  { \
    expression \
  }

#else // RELEASE
#include "Tools/Debugging/Debugging.h"

/*
 * Allows for the measurement of time 
 * @param eventID The id of the stop watch
 * @param expression The expression of which the execution time is measured
 */
#define STOP_TIME_ON_REQUEST(eventID, expression) \
  DEBUG_RESPONSE("stopwatch:" eventID, \
  unsigned _startTimeN; \
  static unsigned _counter = 0; \
  _startTimeN = SystemCall::getRealSystemTime();\
  { expression } \
  OUTPUT(idStopwatch, bin, eventID << _startTimeN << SystemCall::getRealSystemTime() << _counter++) \
  ); \
  DEBUG_RESPONSE_NOT("stopwatch:" eventID, expression );
#endif // RELEASE

#endif //Stopwatch_h
