#include "dns_sd.h"
#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <ctime>
#include <string>

static DNSServiceRef client_browse(0);
static DNSServiceRef client_resolv(0);

static void DNSSD_API handle_resolv_reply(DNSServiceRef sdRef,
                                          DNSServiceFlags flags,
                                          uint32_t ifIndex,
                                          DNSServiceErrorType errorCode,
                                          const char *fullname,
                                          uint16_t rrtype,
                                          uint16_t rrclass,
                                          uint16_t rdlen,
                                          const void *rdata,
                                          uint32_t ttl,
                                          void *context)
{
  (void)sdRef;
  (void)ifIndex;
  (void)errorCode;
  (void)ttl;
  (void)context;

  //handle added IN A
  if ((kDNSServiceType_A == rrtype) && (flags & kDNSServiceFlagsAdd) && rdlen > 3 )
  {
    const unsigned char* rd = (const unsigned char*) rdata;
    std::printf("%s has address %d.%d.%d.%d\n", fullname, rd[0], rd[1], rd[2], rd[3]);
    fflush( stdout );
  }
}


static void DNSSD_API handle_browse_reply(DNSServiceRef clientf,
                                          DNSServiceFlags flags,
                                          uint32_t ifIndex,
                                          DNSServiceErrorType errorCode,
                                          const char *replyName,
                                          const char *replyType,
                                          const char *replyDomain,
                                          void *context)
{
  (void)clientf;
  (void)errorCode;
  (void)context;
  if (flags & kDNSServiceFlagsAdd)
  {
    char queryName[128];
    sprintf_s(queryName, 128, "%s%s", replyName, ".local");
    //found a nao. now query for address.
    if (client_resolv) DNSServiceRefDeallocate(client_resolv);
    DNSServiceQueryRecord(&client_resolv, 0, ifIndex, queryName, kDNSServiceType_A, kDNSServiceClass_IN, handle_resolv_reply, NULL);
  }
  else
  {
    std::printf("%s has gone.\n", replyName);
    std::fflush( stdout );
  }
}


int main(int argc, char **argv)
{
  const std::string service("_bhuman-robot._tcp");
  const std::string domain("");

  DNSServiceErrorType err = DNSServiceBrowse(&client_browse, 0, kDNSServiceInterfaceIndexAny,
                                             service.c_str(), domain.c_str(), handle_browse_reply, 0);
  if(err)
  {
    std::fprintf(stderr, "DNSServiceBrowse failed.\n");
    return(-1);
  }

  while(1)
  {
    //use select to wait for event
    fd_set readfds;
    struct timeval tv = {500000, 0};

    const int fd_browse = client_browse ? DNSServiceRefSockFD(client_browse) : -1;
    const int fd_resolv = client_resolv ? DNSServiceRefSockFD(client_resolv) : -1;

    FD_ZERO(&readfds);

    if (fd_browse > 0) FD_SET(fd_browse , &readfds);
    if (fd_resolv > 0) FD_SET(fd_resolv , &readfds);
    const int max_fd = fd_browse > fd_resolv ? fd_browse : fd_resolv;

    if (select(max_fd + 1, &readfds, 0, 0, &tv) > 0)
    {
      if (fd_browse > 0 && FD_ISSET(fd_browse, &readfds)) DNSServiceProcessResult(client_browse);
      if (fd_resolv > 0 && FD_ISSET(fd_resolv, &readfds)) DNSServiceProcessResult(client_resolv);
    }
    else
    {
      std::perror("select() failed");
      if (errno != EINTR) break;
    }
  }

  if (client_browse) DNSServiceRefDeallocate(client_browse);
  if (client_resolv) DNSServiceRefDeallocate(client_resolv);
  return 0;
}
